# -*- coding: utf-8 -*-
#
# django-codenerix-products
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

from operator import or_
from collections import Iterable
from functools import reduce
from decimal import Decimal

from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models, transaction, IntegrityError
from django.db.models import F, Q
from django.utils import timezone
from django.utils.encoding import smart_text
from django.utils.translation import ugettext_lazy as _
from django.conf import settings

from codenerix.fields import ImageAngularField
from codenerix.helpers import nameunify, qobject_builder_string_search
from codenerix.lib.helpers import upload_path
from codenerix.models import CodenerixModel
from codenerix.fields import WysiwygAngularField
from codenerix_extensions.helpers import get_language_database
from codenerix_extensions.files.models import GenImageFile, GenDocumentFile, GenImageFileNull
from codenerix_storages.models import StorageBox

from codenerix_products.exceptions import ProductUniqueAlreadyExists, ProductUniqueQuantityExceeded, ProductUniqueNotProductFinal, ProductFinalAttributeOnlyOne


CURRENCY_MAX_DIGITS = getattr(settings, 'CDNX_INVOICING_CURRENCY_MAX_DIGITS', 10)
CURRENCY_DECIMAL_PLACES = getattr(settings, 'CDNX_INVOICING_CURRENCY_DECIMAL_PLACES', 2)

PRODUCT_UNIQUE_VALUE_LENGTH = 80

TYPE_PRICE_PERCENTAGE = 'P'
TYPE_PRICE_INCREASE = 'I'
TYPE_PRICE_FINAL = 'F'

TYPE_PRICES = (
    (TYPE_PRICE_PERCENTAGE, _('Porcentaje sobre el precio del producto')),
    (TYPE_PRICE_INCREASE, _('Incremento sobre el precio del producto')),
    (TYPE_PRICE_FINAL, _('Precio final')),
)

TYPE_VALUE_LIST = 'O'
TYPE_VALUE_BOOLEAN = 'B'
TYPE_VALUE_FREE = 'F'
TYPE_VALUES = (
    (TYPE_VALUE_FREE, _('Sin validacion')),
    (TYPE_VALUE_BOOLEAN, _('Boolean')),
    (TYPE_VALUE_LIST, _('Lista de opciones')),
)

TYPE_ORIENTAZION = (
    ('R', _('Right')),
    ('L', _('Left')),
)


# Model Mixins

class CustomQueryMixin(object):

    @classmethod
    def query_or(cls, query, *values_list, **annotations):
        pop_annotations = False
        if 'pop_annotations' in annotations:
            pop_annotations = annotations['pop_annotations']
            annotations.pop('pop_annotations')

        annotated_keys = annotations.values()
        annotations = {key: F(value) for key, value in annotations.items()}

        if isinstance(query, Iterable):
            query = reduce(or_, query)

        result = cls.objects.filter(query).values(*values_list).annotate(**annotations)

        if pop_annotations:
            for querydict in result:
                for value in annotated_keys:
                    querydict.pop(value)

        return result


# tipos de impuestos aplicables a los productos
class TypeTax(CodenerixModel):
    tax = models.FloatField(_("Tax (%)"), validators=[MinValueValidator(0), MaxValueValidator(100)], blank=False, null=False)
    name = models.CharField(_("Name"), max_length=250, blank=False, null=False, unique=True)
    default = models.BooleanField(_("Default"), blank=False, default=False)
    recargo_equivalencia = models.FloatField(_("Recargo de equivalencia (%)"), validators=[MinValueValidator(0), MaxValueValidator(100)], blank=False, null=False)

    def __unicode__(self):
        return u"{}".format(smart_text(self.name))

    def __str__(self):
        return self.__unicode__()

    def __fields__(self, info):
        fields = []
        fields.append(('name', _("Name")))
        fields.append(('tax', _("Tax (%)")))
        fields.append(('recargo_equivalencia', _("Recargo de equivalencia (%)")))
        fields.append(('default', _('Default')))
        return fields

    def lock_delete(self):
        if self.products.exists():
            return _("Cannot delete type tax model, relationship between type tax model and products")
        else:
            return super(TypeTax, self).lock_delete()

    def save(self, *args, **kwargs):
        with transaction.atomic():
            if self.default:
                TypeTax.objects.exclude(pk=self.pk).update(default=False)
            else:
                if not TypeTax.objects.exclude(pk=self.pk).filter(default=True).exists():
                    self.default = True

        if self.pk:
            obj = TypeTax.objects.get(pk=self.pk)
            if obj.tax != self.tax:
                result = super(TypeTax, self).save(*args, **kwargs)
                for product in self.products.all():
                    for pf in product.products_final.all():
                        pf.recalculate()
            else:
                result = super(TypeTax, self).save(*args, **kwargs)
        else:
            result = super(TypeTax, self).save(*args, **kwargs)
        return result


# atributos
class GenAttr(CodenerixModel, GenImageFileNull):  # META: Abstract class
    """
    type_value:
        * libre (no hay validación extra)
        * Boolean (no/si)
        * lista de opciones (select)
    type_price:
        * porcentaje sobre el precio del producto
        * incremento sobre el precio del producto
        * precio final
    """
    class Meta(CodenerixModel.Meta):
        abstract = True

    type_value = models.CharField(_("Type value"), max_length=2, choices=TYPE_VALUES, blank=False, null=False, default=TYPE_VALUE_FREE)
    price = models.FloatField(_("Price"), blank=False, null=False, default=0)
    type_price = models.CharField(_("Type price"), max_length=2, choices=TYPE_PRICES, blank=False, null=False, default=TYPE_PRICE_PERCENTAGE)
    public = models.BooleanField(_("Public"), blank=True, null=False, default=True)
    order = models.SmallIntegerField(_("Order"), blank=True, null=True)

    def __fields__(self, info):
        fields = []
        fields.append(('family', _('Family')))
        fields.append(('category', _('Category')))
        fields.append(('get_type_value_display', _('Type Value')))
        fields.append(('public', _('Public')))
        fields.append(('order', _('Order')))

        lang = settings.LANGUAGES_DATABASES[0].lower()
        fields.append(('{}__description'.format(lang), 'Description', 100))
        return fields

    def __unicode__(self):
        return u"{}".format(getattr(self, settings.LANGUAGES_DATABASES[0].lower()).description)

    def __str__(self):
        return self.__unicode__()


# description del texto en diferentes idiomas
class GenText(CodenerixModel):  # META: Abstract class
    class Meta(CodenerixModel.Meta):
        abstract = True

    description = models.CharField(_("Description"), max_length=250, blank=False, null=False)

    def __fields__(self, info):
        fields = []
        fields.append(('description', _('Description'), 100))
        return fields

    def __unicode__(self):
        return u"{}".format(self.description)

    def __str__(self):
        return self.__unicode__()


# texto de posicionamiento
class GenSEOText(CodenerixModel):  # META: Abstract class
    class Meta(CodenerixModel.Meta):
        abstract = True

    meta_title = models.CharField(_("Meta Title"), max_length=70, blank=True, null=True)
    meta_description = models.CharField(_("Meta Description"), max_length=70, blank=True, null=True)
    meta_keywords = models.CharField(_("Meta Keywords"), max_length=160, blank=True, null=True)


# description del texto en diferentes idiomas
class GenTextSlug(GenSEOText):  # META: Abstract class
    class Meta(CodenerixModel.Meta):
        abstract = True

    slug = models.CharField(_("Slug"), max_length=250, blank=False, null=False, unique=True)
    name = models.CharField(_("Name"), max_length=250, blank=False, null=False)
    description = WysiwygAngularField(_("Description"), blank=False, null=False)

    def __fields__(self, info):
        fields = []
        fields.append(('name', _('Name'), 100))
        fields.append(('description', _('Description'), 100))
        return fields

    def __unicode__(self):
        return u"{}".format(self.name)

    def __str__(self):
        return self.__unicode__()

    def save(self, *args, **kwards):
        self.slug = nameunify(self.slug, True)
        return super(GenTextSlug, self).save(*args, **kwards)


class GenTextTitle(CodenerixModel):  # META: Abstract class
    class Meta(CodenerixModel.Meta):
        abstract = True

    title = models.CharField(_("Text alternavive image"), max_length=250, blank=False, null=False)
    description = WysiwygAngularField(_("Description"), blank=True, null=True)


# texto de los product y brands
class GenProductBrandText(GenSEOText):  # META: Abstract class
    class Meta(CodenerixModel.Meta):
        abstract = True

    description_short = WysiwygAngularField(_("Description short"), blank=True, null=True)
    description_long = WysiwygAngularField(_("Description long"), blank=True, null=True)
    # url amigable
    slug = models.CharField(_("Slug"), max_length=250, blank=False, null=False, unique=True)
    name = models.CharField(_("Name"), max_length=250, blank=False, null=False)
    public = models.BooleanField(_("Public"), blank=True, null=False, default=True)

    def __str__(self):
        return u"{}".format(smart_text(self.description_short))

    def __unicode__(self):
        return self.__str__()

    def __fields__(self, info):
        fields = []
        fields.append(('name', _("Name")))
        fields.append(('meta_title', _("Tittle")))
        fields.append(('meta_description', _("Meta Description")))
        fields.append(('description_short', _("Description short")))
        fields.append(('description_long', _("Description long")))
        fields.append(('slug', _("Slug")))
        return fields

    def save(self, *args, **kwards):
        self.slug = nameunify(self.slug, True)
        return super(GenProductBrandText, self).save(*args, **kwards)


class GenProductText(GenProductBrandText):  # META: Abstract class
    class Meta(CodenerixModel.Meta):
        abstract = True

    tags = models.TextField(_("TAGs"), blank=True, null=True)


class GenBrandText(GenProductBrandText):  # META: Abstract class
    class Meta(CodenerixModel.Meta):
        abstract = True


# texto de productfinal
class GenProductFinalText(GenProductText):  # META: Abstract class
    class Meta(CodenerixModel.Meta):
        abstract = True

    description_sample = WysiwygAngularField(_("Sample description"), blank=True, null=True)

# #####################################


# familias
class Family(CodenerixModel, GenImageFileNull):
    code = models.CharField(_("Code"), max_length=250, blank=True, null=True, unique=True)
    public = models.BooleanField(_("Public"), blank=True, null=False, default=True)
    order = models.SmallIntegerField(_("Order"), blank=True, null=True)
    show_menu = models.BooleanField(_("Show menu"), blank=True, null=False, default=True)
    icon = ImageAngularField(_("Icon"), upload_to=upload_path, max_length=200, blank=True, null=True, help_text=_(u'Se aconseja que sea una imagen superior a 200px transparente y en formato png o svg'))

    def __fields__(self, info):
        fields = []
        fields.append(('order', _("Order")))
        fields.append(('{}__name'.format(settings.LANGUAGES_DATABASES[0].lower()), _("Name")))
        fields.append(('code', _("Code")))
        fields.append(('public', _("Public")))
        fields.append(('show_menu', _("Show Menu")))
        return fields

    def __str__(self):
        if self.code:
            return u"{} ({})".format(smart_text(getattr(self, settings.LANGUAGES_DATABASES[0].lower()).name), smart_text(self.code))
        else:
            return u"{}".format(smart_text(getattr(self, settings.LANGUAGES_DATABASES[0].lower()).name))

    def __unicode__(self):
        return self.__str__()

    def lock_delete(self):
        if self.products.exists():
            return _("Cannot delete family model, relationship between family model and products")
        elif self.features.exists():
            return _("Cannot delete family model, relationship between family model and features")
        elif self.attributes.exists():
            return _("Cannot delete family model, relationship between family model and attributes")
        elif self.feature_specials.exists():
            return _("Cannot delete family model, relationship between family model and feature special")
        elif self.categories.exists():
            return _("Cannot delete family model, relationship between family model and categories")
        else:
            return super(Family, self).lock_delete()


# categorias
class Category(CodenerixModel):
    code = models.CharField(_("Code"), max_length=250, blank=True, null=True, unique=True)
    family = models.ForeignKey(Family, on_delete=models.CASCADE, related_name='categories', verbose_name=_("Family"))
    public = models.BooleanField(_("Public"), blank=True, null=False, default=True)
    show_menu = models.BooleanField(_("Show menu"), blank=True, null=False, default=True)
    show_only_product_stock = models.BooleanField(_("Show only products in stock"), blank=True, null=False, default=True)
    image = ImageAngularField(_("Image"), upload_to=upload_path, max_length=200, blank=True, null=True, help_text=_(u'Se aconseja un tamaño comprendido entre 1200px y 2000px'))
    icon = ImageAngularField(_("Icon"), upload_to=upload_path, max_length=200, blank=True, null=True, help_text=_(u'Se aconseja que sea una imagen superior a 200px transparente y en formato png o svg'))
    order = models.SmallIntegerField(_("Order"), blank=True, null=True)

    def __str__(self):
        if self.code:
            return u"{} ({})".format(smart_text(getattr(self, settings.LANGUAGES_DATABASES[0].lower()).name), smart_text(self.code))
        else:
            return u"{}".format(smart_text(getattr(self, settings.LANGUAGES_DATABASES[0].lower()).name))

    def __unicode__(self):
        return self.__str__()

    def __fields__(self, info):
        fields = []
        fields.append(('order', _("Order")))
        fields.append(('{}__name'.format(settings.LANGUAGES_DATABASES[0].lower()), _("Name")))
        fields.append(('code', _("Code")))
        fields.append(('family__{}__name'.format(settings.LANGUAGES_DATABASES[0].lower()), _("Family name")))
        fields.append(('family__code', _("Family code")))
        fields.append(('public', _("Public")))
        fields.append(('show_menu', _("Show Menu")))
        fields.append(('show_only_product_stock', _("Show Only Products with Stock")))
        return fields

    def lock_delete(self):
        if self.products.exists():
            return _("Cannot delete category model, relationship between category model and products")
        elif self.features.exists():
            return _("Cannot delete category model, relationship between category model and features")
        elif self.attributes.exists():
            return _("Cannot delete category model, relationship between category model and attributes")
        elif self.feature_specials.exists():
            return _("Cannot delete category model, relationship between category model and feature special")
        elif self.subcategory.exists():
            return _("Cannot delete category model, relationship between category model and subcategories")
        else:
            return super(Category, self).lock_delete()


# subcategorias
class Subcategory(CodenerixModel):
    code = models.CharField(_("Code"), max_length=250, blank=True, null=True, unique=True)
    category = models.ForeignKey(Category, on_delete=models.CASCADE, related_name='subcategory', verbose_name=_("Category"))
    public = models.BooleanField(_("Public"), blank=True, null=False, default=True)
    show_menu = models.BooleanField(_("Show menu"), blank=True, null=False, default=True)
    show_brand = models.BooleanField(_("Show brand (for menu)"), blank=True, null=False, default=True)
    outstanding = models.BooleanField(_("Outstanding"), blank=True, null=False, default=False)
    order = models.SmallIntegerField(_("Order"), blank=True, null=True)
    image = ImageAngularField(_("Image"), upload_to=upload_path, max_length=200, blank=True, null=True, help_text=_(u'Se aconseja un tamaño comprendido entre 1200px y 2000px'))
    icon = ImageAngularField(_("Icon"), upload_to=upload_path, max_length=200, blank=True, null=True, help_text=_(u'Se aconseja que sea una imagen superior a 200px transparente y en formato png o svg'))

    def __str__(self):
        if self.code:
            return u"{} ({})".format(smart_text(getattr(self, settings.LANGUAGES_DATABASES[0].lower()).name), smart_text(self.code))
        else:
            return u"{}".format(smart_text(getattr(self, settings.LANGUAGES_DATABASES[0].lower()).name))

    def __unicode__(self):
        return self.__str__()

    def __fields__(self, info):
        fields = []
        fields.append(('{}__name'.format(settings.LANGUAGES_DATABASES[0].lower()), _("Name")))
        fields.append(('code', _("Code")))
        fields.append(('category__{}__name'.format(settings.LANGUAGES_DATABASES[0].lower()), _("Category name")))
        fields.append(('category__code', _("Category code")))
        fields.append(('category__family__{}__name'.format(settings.LANGUAGES_DATABASES[0].lower()), _("Family name")))
        fields.append(('category__family__code', _("Family code")))
        fields.append(('public', _("Public")))
        fields.append(('show_menu', _("Show Menu")))
        fields.append(('show_brand', _("Show brand (for menu)")))
        fields.append(('outstanding', _("Outstanding")))
        fields.append(('order', _("Order")))
        return fields

    def lock_delete(self):
        if self.products.exists():
            return _("Cannot delete subcategory model, relationship between subcategory model and products")
        else:
            return super(Subcategory, self).lock_delete()


# grupo de valores
class GroupValues(CodenerixModel):  # META: Abstract class
    class Meta(CodenerixModel.Meta):
        abstract = True

    name = models.CharField(_("Name"), max_length=250, blank=True, null=True, unique=True)

    def __fields__(self, info):
        fields = []
        fields.append(('name', _("Name")))
        return fields

    def __unicode__(self):
        return u"{}".format(smart_text(self.name))

    def __str__(self):
        return self.__unicode__()


# grupo de valores para features
class GroupValueFeature(GroupValues):
    def __fields__(self, info):
        fields = super(GroupValueFeature, self).__fields__(info)
        fields.append(('options_value_feature', _('Option value')))
        return fields

    def lock_delete(self):
        if self.options_value_feature.exists():
            return _("Cannot delete group value model, relationship between group value model and options")
        elif self.features.exists():
            return _("Cannot delete group value model, relationship between group value model and features")
        else:
            return super(GroupValueFeature, self).lock_delete()


# grupo de valores para attributes
class GroupValueAttribute(GroupValues):
    def __fields__(self, info):
        fields = super(GroupValueAttribute, self).__fields__(info)
        fields.append(('options_value_attribute', _('Option value')))
        return fields

    def lock_delete(self):
        if self.options_value_attribute.exists():
            return _("Cannot delete group value model, relationship between group value model and options")

        elif self.attributes.exists():
            return _("Cannot delete group value model, relationship between group value model and attributes")
        else:
            return super(GroupValueAttribute, self).lock_delete()


# grupo de valores para features special
class GroupValueFeatureSpecial(GroupValues):
    def __fields__(self, info):
        fields = super(GroupValueFeatureSpecial, self).__fields__(info)
        fields.append(('options_value_feature_special', _('Option value')))
        return fields

    def lock_delete(self):
        if self.options_value_feature_special.exists():
            return _("Cannot delete group value model, relationship between group value model and options")
        elif self.feature_specials.exists():
            return _("Cannot delete group value model, relationship between group value model and feature special")
        else:
            return super(GroupValueFeatureSpecial, self).lock_delete()


# opciones de los grupos de valores
class OptionValues(CodenerixModel):  # META: Abstract class
    class Meta(CodenerixModel.Meta):
        abstract = True

    def __fields__(self, info):
        fields = []
        for lang_code in settings.LANGUAGES_DATABASES:
            fields.append(('{}__description'.format(lang_code.lower()), _(lang_code.upper())))
        return fields

    def __unicode__(self):
        string = []
        for lang_code in settings.LANGUAGES_DATABASES:
            string.append(u"{}: {}".format(lang_code, getattr(self, lang_code.lower())))

        return u"{}".format(" | ".join(string))

    def __str__(self):
        return self.__unicode__()


# opciones de los grupos de valores para features
class OptionValueFeature(OptionValues):
    group = models.ForeignKey(GroupValueFeature, on_delete=models.CASCADE, related_name='options_value_feature', verbose_name=_("Options value"))


# opciones de los grupos de valores para attributes
class OptionValueAttribute(OptionValues):
    group = models.ForeignKey(GroupValueAttribute, on_delete=models.CASCADE, related_name='options_value_attribute', verbose_name=_("Options value"))


# opciones de los grupos de valores features special
class OptionValueFeatureSpecial(OptionValues):
    group = models.ForeignKey(GroupValueFeatureSpecial, on_delete=models.CASCADE, related_name='options_value_feature_special', verbose_name=_("Options value"))


# caracteristicas (comunes a todos los productos (resolución, RAM))
class Feature(GenAttr):
    family = models.ForeignKey(Family, on_delete=models.CASCADE, related_name='features', verbose_name=_("Family"), blank=True, null=True)
    category = models.ForeignKey(Category, on_delete=models.CASCADE, related_name='features', verbose_name=_("Category"), blank=True, null=True)
    list_value = models.ForeignKey(GroupValueFeature, on_delete=models.CASCADE, related_name='features', verbose_name=_("List value"), blank=True, null=True)

    def save(self, *args, **kwargs):
        if self.pk:
            obj = Feature.objects.get(pk=self.pk)
            if obj.price != self.price:
                result = super(Feature, self).save(*args, **kwargs)
                for pf in ProductFinal.objects.filter(product__product_features__feature=self):
                    pf.recalculate()
            else:
                result = super(Feature, self).save(*args, **kwargs)
        else:
            result = super(Feature, self).save(*args, **kwargs)
        return result

    def lock_delete(self):
        if self.product_features.exists():
            return _("Cannot delete feature model, relationship between feature model and products")
        else:
            return super(Feature, self).lock_delete()


# atributos (subconjunto de productos (color, talla))
class Attribute(GenAttr):
    """
    los atributos están relacionados entre ellos
    el order indica el orden en el que aparcerán
    """
    family = models.ForeignKey(Family, on_delete=models.CASCADE, related_name='attributes', verbose_name=_("Family"), blank=True, null=True)
    category = models.ForeignKey(Category, on_delete=models.CASCADE, related_name='attributes', verbose_name=_("Category"), blank=True, null=True)
    attribute = models.ForeignKey("self", on_delete=models.CASCADE, related_name='attributes', verbose_name=_("Attribute"), blank=True, null=True)
    list_value = models.ForeignKey(GroupValueAttribute, on_delete=models.CASCADE, related_name='attributes', verbose_name=_("List value"), blank=True, null=True)

    def save(self, *args, **kwargs):
        if self.pk:
            obj = Attribute.objects.get(pk=self.pk)
            if obj.price != self.price:
                result = super(Attribute, self).save(*args, **kwargs)
                for pf in ProductFinal.objects.filter(products_final_attr__attribute=self):
                    pf.recalculate()
            else:
                result = super(Attribute, self).save(*args, **kwargs)
        else:
            result = super(Attribute, self).save(*args, **kwargs)
        return result

    def lock_delete(self):
        if self.products_final_attr.exists():
            return _("Cannot delete attribute model, relationship between attribute model and products final")
        else:
            return super(Attribute, self).lock_delete()


# caracteristicas especiales (unico por producto (imei, fecha caducidad))
class FeatureSpecial(GenAttr):
    family = models.ForeignKey(Family, on_delete=models.CASCADE, related_name='feature_specials', verbose_name=_("Family"), blank=True, null=True)
    category = models.ForeignKey(Category, on_delete=models.CASCADE, related_name='feature_specials', verbose_name=_("Category"), blank=True, null=True)
    list_value = models.ForeignKey(GroupValueFeatureSpecial, on_delete=models.CASCADE, related_name='feature_specials', verbose_name=_("List value"), blank=True, null=True)
    unique = models.BooleanField(_("The value must unique"), blank=True, null=False, default=True)

    def __fields__(self, info):
        fields = super(FeatureSpecial, self).__fields__(info)
        fields.append(('unique', _("The value must unique")))
        return fields

    def save(self, *args, **kwargs):
        if self.pk:
            obj = FeatureSpecial.objects.get(pk=self.pk)
            if obj.price != self.price:
                result = super(FeatureSpecial, self).save(*args, **kwargs)
                for pf in ProductFinal.objects.filter(product__feature_special=self):
                    pf.recalculate()
            else:
                result = super(FeatureSpecial, self).save(*args, **kwargs)
        else:
            result = super(FeatureSpecial, self).save(*args, **kwargs)
        return result

    def lock_delete(self):
        if self.products.exists():
            return _("Cannot delete feature special model, relationship between feature special model and products")
        else:
            return super(FeatureSpecial, self).lock_delete()


# Marcas
class Brand(CodenerixModel, GenImageFileNull):
    outstanding = models.BooleanField(_("Outstanding"), blank=True, null=False, default=True)
    order = models.SmallIntegerField(_("Order"), blank=True, null=True)
    show_menu = models.BooleanField(_("Show menu"), blank=True, null=False, default=True)

    def __unicode__(self):
        name_res = _("Missing brand name")
        for lang_code, lang_name in settings.LANGUAGES:
            translation = getattr(self, lang_code.lower(), None)
            if translation is not None:
                if translation.name and len(translation.name) > 0:
                    name_res = u"{}".format(smart_text(translation.name))
                else:
                    name_res = u"{}".format(smart_text(translation.slug))
        return name_res

    def __str__(self):
        return self.__unicode__()

    def __fields__(self, info):
        fields = []
        fields.append(('outstanding', _("Outstanding")))
        fields.append(('order', _("Order")))
        fields.append(('show_menu', _("Show menu")))
        return fields

    def lock_delete(self):
        if self.products.exists():
            return _("Cannot delete brand model, relationship between brand model and products")
        else:
            return super(Brand, self).lock_delete()


# definición de productos
class GenProduct(CodenerixModel):  # META: Abstract class
    """
    @warning: brand no debe permitir nulo. Recordar para la limpieza de la bbdd.
    """
    # se hace abstracta para que hereden product y productfinal
    class Meta(CodenerixModel.Meta):
        abstract = True

    model = models.CharField(_("Model"), max_length=250, blank=True, null=True)
    brand = models.ForeignKey(Brand, on_delete=models.CASCADE, related_name='products', verbose_name=_("Brand"), blank=True, null=True)
    tax = models.ForeignKey(TypeTax, on_delete=models.CASCADE, related_name='products', verbose_name=_("Tax (%)"), null=True)
    family = models.ForeignKey(Family, on_delete=models.CASCADE, related_name='products', verbose_name=_("Family"))
    category = models.ForeignKey(Category, on_delete=models.CASCADE, related_name='products', verbose_name=_("Category"))
    subcategory = models.ForeignKey(Subcategory, on_delete=models.CASCADE, related_name='products', verbose_name=_("Subcategory"))
    public = models.BooleanField(_("Public"), blank=True, null=False, default=True)
    code = models.CharField(_("Code"), max_length=250, blank=False, null=False, unique=True)
    price_base = models.DecimalField(_("Price base"), blank=False, null=False, max_digits=CURRENCY_MAX_DIGITS, decimal_places=CURRENCY_DECIMAL_PLACES, default=0)
    # producto para la venta
    of_sales = models.BooleanField(_("Sales"), blank=True, null=False, default=True)
    # producto para la compra
    of_purchase = models.BooleanField(_("Purchase"), blank=True, null=False, default=True)
    # es necesario que el producto tenga stock para su venta
    force_stock = models.BooleanField(_("Force stock"), blank=True, null=False, default=True)
    url_video = models.CharField(_("Url Video"), max_length=250, blank=True, null=True)
    # indica si es necesario tener una caracteristica especial obligatoriamente
    feature_special = models.ForeignKey(FeatureSpecial, on_delete=models.CASCADE, related_name='products', verbose_name=_("Feature special"), blank=True, null=True)
    packing_cost = models.DecimalField(_("Packing cost"), blank=False, null=False, max_digits=CURRENCY_MAX_DIGITS, decimal_places=CURRENCY_DECIMAL_PLACES, default=0)
    weight = models.FloatField(_("Weight"), blank=False, null=False, default=0)
    caducable = models.BooleanField(_("Caducable"), blank=True, null=False, default=False)

    def __unicode__(self):
        return u"{}".format(smart_text(self.code))

    def __str__(self):
        return self.__unicode__()

    def __fields__(self, info):
        lang = get_language_database()

        fields = []
        fields.append(('family__{}__name'.format(lang), _("Family")))
        fields.append(('category__{}__name'.format(lang), _("Category")))
        fields.append(('subcategory__{}__name'.format(lang), _("Subcategory")))
        fields.append(('public', _("Public")))
        fields.append(('tax', _("Tax")))
        fields.append(('code', _("Code")))
        fields.append(('price_base', _("Price base")))
        fields.append(('of_sales', _("Sales")))
        fields.append(('of_purchase', _("Purchase")))
        fields.append(('force_stock', _("Force stock")))
        fields.append(('url_video', _("Url Video")))
        fields.append(('feature_special', _("Feature special")))
        fields.append(('packing_cost', _("Packing cost")))
        fields.append(('weight', _("Weight")))
        fields.append(('caducable', _("Caducable")))

        return fields

    def __searchQ__(self, info, text):
        lang = get_language_database()

        text_filters = {}
        text_filters['product_slug'] = Q(**{"{}__slug__icontains".format(lang): text})
        text_filters['product_name'] = Q(**{"{}__name__icontains".format(lang): text})
        try:
            text_filters['identifier'] = Q(pk=int(text))
        except ValueError:
            pass
        return text_filters


# productos
class Product(CustomQueryMixin, GenProduct):

    def lock_delete(self):
        if self.products_final.exists():
            return _("Cannot delete product model, relationship between product model and products final")
        elif self.products_image.exists():
            return _("Cannot delete product model, relationship between product model and products image")
        elif self.products_document.exists():
            return _("Cannot delete product model, relationship between product model and products document")
        elif self.product_features.exists():
            return _("Cannot delete product model, relationship between product model and products feature")
        elif self.products_related.exists():
            return _("Cannot delete product model, relationship between product model and products related")
        elif self.products_related_sold.exists():
            return _("Cannot delete product model, relationship between product model and products related sold")
        else:
            return super(Product, self).lock_delete()

    def save(self, *args, **kwargs):
        if self.pk:
            obj = Product.objects.get(pk=self.pk)
            if obj.price_base != self.price_base or obj.tax != self.tax:
                result = super(Product, self).save(*args, **kwargs)
                for pf in self.products_final.all():
                    pf.recalculate()
            else:
                result = super(Product, self).save(*args, **kwargs)
        else:
            result = super(Product, self).save(*args, **kwargs)
        return result

    def pass_to_productfinal(self, ean13=None):
        get_class = lambda x: globals()[x]

        try:
            with transaction.atomic():
                pf = ProductFinal()
                pf.product = self
                pf.code = self.code
                pf.price_base_local = self.price_base
                if ean13:
                    pf.ean13 = ean13
                pf.save()

                for lang_code in settings.LANGUAGES_DATABASES:
                    lang = getattr(self, lang_code.lower(), None)
                    if lang:
                        model_name = "{}Text{}".format('ProductFinal', lang_code)
                        model = get_class(model_name)
                        pft = model()
                        pft.product = pf
                        pft.meta_title = getattr(lang, 'meta_title', None)
                        pft.meta_description = getattr(lang, 'meta_description', None)
                        pft.description_short = getattr(lang, 'description_short', None)
                        pft.description_long = getattr(lang, 'description_long', None)
                        pft.slug = getattr(lang, 'slug', None)
                        pft.name = getattr(lang, 'name', None)
                        pft.public = getattr(lang, 'public', None)
                        pft.meta_title = getattr(lang, 'meta_title', None)
                        pft.meta_description = getattr(lang, 'meta_description', None)
                        pft.meta_keywords = getattr(lang, 'meta_keywords', None)
                        pft.tags = getattr(lang, 'tags', None)
                        pft.save()
        except IntegrityError as e:
            raise IntegrityError(e)

    @classmethod
    def find_product(cls, query, lang, onlypublic=False):
        product = cls.query_or(
            query,
            "pk",
            "price_base",
            # "product__pk",
            "model",
            "code",
            "url_video",
            # "product__brand__{}__name".format(lang),
            # "product__brand__{}__slug".format(lang),
            # "product__brand__image",
            # "product__brand__outstanding",
            # "product__family",
            # "product__family__code",
            "family__pk".format(lang),
            "family__{}__slug".format(lang),
            # "product__family__image",
            "family__{}__name".format(lang),
            "family__{}__description".format(lang),
            "category__pk",
            # "product__category",
            # "product__category__code".format(lang),
            # "product__category__image".format(lang),
            "category__{}__slug".format(lang),
            "category__{}__name".format(lang),
            "category__{}__description".format(lang),
            # "product__subcategory",
            # "product__subcategory__code",
            # "product__subcategory__image",
            "subcategory__pk",
            "subcategory__{}__slug".format(lang),
            "subcategory__{}__name".format(lang),
            "subcategory__{}__description".format(lang),
            "family__image",
            "family__icon",
            "category__image",
            "subcategory__image",
            "tax__tax",
            "{}__meta_title".format(lang),
            "{}__meta_description".format(lang),
            "{}__meta_keywords".format(lang),
            "{}__description_short".format(lang),
            "{}__description_long".format(lang),
            "{}__slug".format(lang),
            "{}__name".format(lang),
            "weight",
            family_image="family__image".format(lang),
            family_name="family__{}__name".format(lang),
            family_slug="family__{}__slug".format(lang),
            family_description="family__{}__description".format(lang),
            category_image="category__image".format(lang),
            category_slug="category__{}__slug".format(lang),
            category_name="category__{}__name".format(lang),
            category_description="category__{}__description".format(lang),
            subcategory_image="subcategory__image".format(lang),
            subcategory_slug="subcategory__{}__slug".format(lang),
            subcategory_name="subcategory__{}__name".format(lang),
            subcategory_description="subcategory__{}__description".format(lang),
            tax="tax__tax",
            meta_title="{}__meta_title".format(lang),
            meta_description="{}__meta_description".format(lang),
            meta_keywords="{}__meta_keywords".format(lang),
            description_short="{}__description_short".format(lang),
            description_long="{}__description_long".format(lang),
            name="{}__name".format(lang),
            slug="{}__slug".format(lang),
            pop_annotations=True
        )
        if onlypublic:
            product = product.exclude(public=False)
        product = product.first()

        # if product:
        #     product_final = cls.objects.get(pk=product['pk'])
        #     prices = product_final.calculate_price()
        #     product['price'] = prices['price_total']

        return product


# productos relacionados mas vendidos
class ProductRelationSold(CodenerixModel):
    product = models.ForeignKey(Product, on_delete=models.CASCADE, blank=False, null=False, related_name='products_related', verbose_name=_("Product"))
    related = models.ForeignKey(Product, on_delete=models.CASCADE, blank=False, null=False, related_name='products_related_sold', verbose_name=_("Products related"))
    hits = models.SmallIntegerField(_("Hits"), blank=True, null=True)

    class Meta(CodenerixModel.Meta):
        unique_together = (('product', 'related'), )

    def __unicode__(self):
        return u"{} ({})".format(smart_text(self.product), smart_text(self.hits))

    def __str__(self):
        return self.__unicode__()

    def __fields__(self, info):
        fields = []
        fields.append(('product', _("Product")))
        fields.append(('related', _("Products related")))
        fields.append(('hits', _("Hits")))
        return fields


# imagenes de productos
class ProductImage(CodenerixModel, GenImageFile):
    product = models.ForeignKey(Product, on_delete=models.CASCADE, related_name='products_image', verbose_name=_("Product"))
    order = models.SmallIntegerField(_("Order"), blank=True, null=True)
    public = models.BooleanField(_("Public"), blank=True, null=False, default=True)
    principal = models.BooleanField(_("Principal"), blank=False, null=False, default=False)
    flagship_product = models.BooleanField(_("Flagship product"), default=False)
    outstanding = models.BooleanField(_("Outstanding"), default=False)

    def __unicode__(self):
        return u"{} ({})".format(smart_text(self.product), smart_text(self.order))

    def __str__(self):
        return self.__unicode__()

    def __fields__(self, info):
        fields = []
        fields.append(('product', _("Product")))
        fields.append(('order', _("Order")))
        fields.append(('public', _("Public")))
        fields.append(('principal', _("Principal")))
        return fields

    # Save necesita un check que indique si debe comprobar o no los productos destacados y productos estrella.
    @transaction.atomic
    def save(self, *args, **kwards):
        if self.principal:
            ProductImage.objects.filter(product=self.product).exclude(pk=self.pk).update(principal=False)
        elif not ProductImage.objects.exclude(pk=self.pk).filter(principal=True).exists():
            self.principal = True

        # Si no hay ninguna imagen con el check de producto estrella, marco esta.
        if self.flagship_product:
            ProductImage.objects.filter(product=self.product, flagship_product=True).exclude(pk=self.pk).update(flagship_product=False)
        elif not ProductImage.objects.filter(product=self.product, flagship_product=True).exclude(pk=self.pk).exists():
            self.flagship_product = True

        # Producto destacado
        if self.outstanding:
            ProductImage.objects.filter(product=self.product, outstanding=True).exclude(pk=self.pk).update(outstanding=False)
        elif not ProductImage.objects.filter(product=self.product, outstanding=True).exclude(pk=self.pk).exists():
            self.outstanding = True

        return super(ProductImage, self).save(*args, **kwards)


# documentos de productos
class ProductDocument(CodenerixModel, GenDocumentFile):
    product = models.ForeignKey(Product, on_delete=models.CASCADE, related_name='products_document', verbose_name=_("Product"))
    public = models.BooleanField(_("Public"), blank=False, null=False, default=False)

    def __unicode__(self):
        return u"{}".format(smart_text(self.product))

    def __str__(self):
        return self.__unicode__()

    def __fields__(self, info):
        fields = []
        fields.append(('product', _("Product")))
        fields.append(('public', _("Public")))
        return fields


# producto final (1 producto muchos atributos) (pulgadas, RAM)
class ProductFinal(CustomQueryMixin, CodenerixModel):
    """
    el stock se relaciona con esta clase
    definición de productos individuales
    """
    product = models.ForeignKey(Product, on_delete=models.CASCADE, blank=False, null=False, related_name='products_final', verbose_name=_('Product'))
    # productos relacionados
    related = models.ManyToManyField("ProductFinal", blank=True, related_name='productsrelated', symmetrical=False)
    related_accesory = models.ManyToManyField("ProductFinal", blank=True, related_name='productsrelatedaccesory', symmetrical=False)
    offer = models.BooleanField(_("Offer"), blank=True, null=False, default=False)
    outstanding = models.BooleanField(_("Outstanding"), blank=True, null=False, default=False)
    most_sold = models.BooleanField(_("Most sold"), blank=True, null=False, default=False)
    # price without tax
    price_base = models.DecimalField(_("Price base"), null=False, blank=False, max_digits=CURRENCY_MAX_DIGITS, decimal_places=CURRENCY_DECIMAL_PLACES, default=0, editable=False)
    # price with tax
    price = models.DecimalField(_("Price"), null=False, blank=False, max_digits=CURRENCY_MAX_DIGITS, decimal_places=CURRENCY_DECIMAL_PLACES, default=0, editable=False)
    ean13 = models.CharField(_("EAN-13"), null=True, blank=True, max_length=13)

    reviews_value = models.FloatField(_("Reviews"), null=False, blank=False, default=0, editable=False)
    reviews_count = models.IntegerField(_("Reviews count"), null=False, blank=False, default=0, editable=False)
    sample = models.BooleanField(_("Sample"), blank=True, null=False, default=False, help_text=_('If this option is checked the product can not be sold'))

    code = models.CharField(_("Code"), max_length=250, blank=True, null=True, unique=True, help_text=_('If it is empty, code is equal to code product'))
    price_base_local = models.DecimalField(_("Price base"), blank=True, null=True, max_digits=CURRENCY_MAX_DIGITS, decimal_places=CURRENCY_DECIMAL_PLACES, help_text=_('If it is empty, price base is equal to price base of product'))

    packing_cost = models.DecimalField(_("Packing cost"), blank=True, null=True, max_digits=CURRENCY_MAX_DIGITS, decimal_places=CURRENCY_DECIMAL_PLACES, help_text=_('If it is empty, packing cost is equal to packing cost of product'))
    weight = models.FloatField(_("Weight"), blank=True, null=True, help_text=_('If it is empty, weight is equal to weight of product'))

    def __str__(self):
        lang = get_language_database()
        lang_model = getattr(self, '{}'.format(lang), None)
        if lang_model:
            name = lang_model.name
        else:
            name = self.product

        if self.ean13:
            name = u"{} ({})".format(smart_text(name), self.ean13)
        else:
            name = u"{}".format(name)
        return name

    def __unicode__(self):
        return self.__str__()

    def __fields__(self, info):
        lang = get_language_database()
        fields = []
        fields.append(('code', _("Code")))
        fields.append(('product__code', _("Product Code")))
        fields.append(('{}__name'.format(lang), _("Product")))
        fields.append(('product__family__{}__name'.format(lang), _("Family")))
        fields.append(('product__category__{}__name'.format(lang), _("Category")))
        fields.append(('product__subcategory__{}__name'.format(lang), _("Subcategory")))
        fields.append(('{}__public'.format(lang), _("Public")))
        fields.append(('price', _("Price")))
        fields.append(('is_pack', _("Is pack")))
        fields.append(('sample', _("Sample")))
        return fields

    def __searchF__(self, info):
        lang = get_language_database()

        fields = {}
        fields['product__family__{}__name'.format(lang)] = (_('Family'), lambda x, lang=lang: Q(**{'product__family__{}__name__icontains'.format(lang): x}), 'input')
        fields['product__category__{}__name'.format(lang)] = (_('Category'), lambda x, lang=lang: Q(**{'product__category__{}__name__icontains'.format(lang): x}), 'input')
        fields['product__subcategory__{}__name'.format(lang)] = (_('Subcategory'), lambda x, lang=lang: Q(**{'product__subcategory__{}__name__icontains'.format(lang): x}), 'input')
        fields['{}__name'.format(lang)] = (_('Product'), lambda x, lang=lang: Q(**{'{}__name__icontains'.format(lang): x}), 'input')
        fields['product__code'] = (_('Product Code'), lambda x, lang=lang: Q(**{'product__code__icontains': x}), 'input')
        return fields

    def __searchQ__(self, info, text):
        lang = get_language_database()

        qobject = qobject_builder_string_search(
            [
                "{}__name".format(lang),
                "{}__slug".format(lang),
            ],
            text
        )

        text_filters = {}
        text_filters['product_name'] = qobject

        return text_filters

    def save(self, *args, **kwards):
        self.recalculate(commit=False)
        return super(ProductFinal, self).save(*args, **kwards)

    def recalculate(self, commit=True):
        prices = self.calculate_price()
        if self.price != prices['price_total'] or self.price_base != prices['price_base']:
            self.price = prices['price_total']
            self.price_base = prices['price_base']
            if commit:
                self.save()

    def lock_delete(self):
        if self.products_final_attr.exists():
            return _("Cannot delete product final model, relationship between product final model and products final attributes")
        elif self.productfinals_image.exists():
            return _("Cannot delete product final model, relationship between product final model and products final image")
        elif self.products_unique.exists():
            return _("Cannot delete product final model, relationship between product final model and products unique")
        elif self.flagship_products.exists():
            return _("Cannot delete product final model, relationship between product final model and products flaship")
        else:
            return super(ProductFinal, self).lock_delete()

    def calculate_price(self):
        if self.price_base_local is None:
            price = self.product.price_base
        else:
            price = self.price_base_local
        tax = self.product.tax.tax
        price_base = price

        # atributos
        update = True
        for attr in self.products_final_attr.all().order_by('-updated'):
            if update:
                if attr.attribute.type_price == TYPE_PRICE_FINAL:
                    price = Decimal(attr.attribute.price)
                    update = False
                elif attr.attribute.type_price == TYPE_PRICE_INCREASE:
                    price += Decimal(attr.attribute.price)
                elif attr.attribute.type_price == TYPE_PRICE_PERCENTAGE:
                    price += price_base * Decimal(attr.attribute.price / 100.0)

        # caracteristicas
        if update:
            for feature in self.product.product_features.all().order_by('-updated'):
                if update:
                    if feature.feature.type_price == TYPE_PRICE_FINAL:
                        price = Decimal(feature.feature.price)
                        update = False
                    elif feature.feature.type_price == TYPE_PRICE_INCREASE:
                        price += Decimal(feature.feature.price)
                    elif feature.feature.type_price == TYPE_PRICE_PERCENTAGE:
                        price += price_base * Decimal(feature.feature.price / 100.0)

        # caracteristicas especiales
        if update and self.product.feature_special:
            if self.product.feature_special.type_price == TYPE_PRICE_FINAL:
                price = Decimal(self.product.feature_special.price)
            elif self.product.feature_special.type_price == TYPE_PRICE_INCREASE:
                price += Decimal(self.product.feature_special.price)
            elif self.product.feature_special.type_price == TYPE_PRICE_PERCENTAGE:
                price += price_base * Decimal(self.product.feature_special.price / 100.0)

        result = {}
        result['price_base'] = price
        result['tax'] = price * Decimal(tax) / 100
        result['price_total'] = price + result['tax']
        return result

    def is_pack(self):
        return self.productfinals_option.exists()

    @property
    def ispack(self):
        return self.is_pack()

    @classmethod
    def get_recommended_products(cls, lang, family=None, category=None, subcategory=None):
        products = []
        query = Q(most_sold=True) | Q(product__products_image__principal=True)
        if family is not None:
            query &= Q(product__family=category)
        if category is not None:
            query &= Q(product__category=category)
        if subcategory is not None:
            query &= Q(product__subcategory=subcategory)
        for product in cls.query_or(
            query,
            "{}__slug".format(lang),
            "offer",
            "created",
            "offer",
            "pk",
            "product__{}__name".format(lang),
            "product__model",
            "product__brand__{}__name".format(lang),
            "product__products_image__image",
            "{}__meta_title".format(lang),
            slug="{}__slug".format(lang),
            meta_title="{}__meta_title".format(lang),
            image="product__products_image__image",
            name="product__{}__name".format(lang),
            pop_annotations=True
        ):
            product['new'] = 1 if (timezone.now() - product['created']).days <= settings.CDNX_PRODUCTS_NOVELTY_DAYS else 0
            products.append(product)

        return products

    @classmethod
    def get_outstanding_products(cls, lang, family=None, category=None, subcategory=None, limit=16):
        products = []
        query = Q(outstanding=True) & (Q(product__products_image__principal=True) | Q(productfinals_image__principal=True))
        if family is not None:
            query &= Q(product__family=family)
        if category is not None:
            query &= Q(product__category=category)
        if subcategory is not None:
            query &= Q(product__subcategory=subcategory)

        qset = cls.objects.filter(
            query
        ).values(
            "{}__slug".format(lang),
            "offer",
            "created",
            "offer",
            "pk",
            "sample",
            "code",
            "product__tax__tax",
            "product__{}__name".format(lang),
            "product__model",
            "product__category__{}__name".format(lang),
            "product__brand__{}__name".format(lang),
            "product__products_image__image",
            "{}__meta_title".format(lang)
        ).annotate(
            slug=F("{}__slug".format(lang)),
            meta_title=F("{}__meta_title".format(lang)),
            image_product=F("product__products_image__image"),
            image_productfinal=F("productfinals_image__image"),
            name=F("product__{}__name".format(lang)),
            category_name=F("product__category__{}__name".format(lang))
        )[:limit]

        for product in qset:
            prices = cls.objects.get(pk=product['pk']).calculate_price()
            product['pop_annotations'] = True
            product['price'] = prices['price_total']
            product['new'] = 1 if (timezone.now() - product['created']).days <= settings.CDNX_PRODUCTS_NOVELTY_DAYS else 0
            if product['image_productfinal']:
                product['image'] = product['image_productfinal']
            else:
                product['image'] = product['image_product']
            products.append(product)

        return products

    @classmethod
    def get_products(cls, lang, family=None, category=None, subcategory=None, brand=None):
        products = []
        query = Q(product__products_image__principal=True)

        if family is not None:
            query &= Q(product__family=family)
        if category is not None:
            query &= Q(product__category=category)
        if subcategory is not None:
            query &= Q(product__subcategory=subcategory)
        if brand is not None:
            query &= Q(product__brand=brand)
        for product in cls.query_or(
            query,
            "{}__slug".format(lang),
            "offer",
            "created",
            "offer",
            "pk",
            "product__tax__tax",
            "product__{}__name".format(lang),
            "product__model",
            "product__brand__{}__name".format(lang),
            "product__products_image__image",
            "{}__meta_title".format(lang),
            slug="{}__slug".format(lang),
            meta_title="{}__meta_title".format(lang),
            image="product__products_image__image",
            name="product__{}__name".format(lang),
            pop_annotations=True
        ):
            prices = cls.objects.get(pk=product['pk']).calculate_price()
            product['price'] = prices['price_total']
            product['new'] = 1 if (timezone.now() - product['created']).days <= settings.CDNX_PRODUCTS_NOVELTY_DAYS else 0
            products.append(product)

        return products

    @classmethod
    def find_product(cls, query, lang, onlypublic=False):
        product = cls.query_or(
            query,
            "pk",
            "offer",
            "outstanding",
            "price",
            "sample",
            "reviews_value",
            "reviews_count",
            "product__pk",
            "product__model",
            "product__code",
            "product__url_video",
            "product__brand__{}__name".format(lang),
            "product__brand__{}__slug".format(lang),
            "product__brand__image",
            "product__brand__outstanding",
            "product__family",
            "product__family__code",
            "product__family__{}__slug".format(lang),
            "product__family__image",
            "product__family__{}__name".format(lang),
            "product__family__{}__description".format(lang),
            "product__category",
            "product__category__code".format(lang),
            "product__category__image".format(lang),
            "product__category__{}__slug".format(lang),
            "product__category__{}__name".format(lang),
            "product__category__{}__description".format(lang),
            "product__subcategory",
            "product__subcategory__code",
            "product__subcategory__image",
            "product__subcategory__{}__slug".format(lang),
            "product__subcategory__{}__name".format(lang),
            "product__subcategory__{}__description".format(lang),
            "product__tax__tax",
            "product__family__image",
            "product__family__icon",
            "product__category__image",
            "product__subcategory__image",
            "{}__meta_title".format(lang),
            "{}__meta_description".format(lang),
            "{}__meta_keywords".format(lang),
            "{}__description_short".format(lang),
            "{}__description_long".format(lang),
            "{}__description_sample".format(lang),
            "{}__slug".format(lang),
            "{}__name".format(lang),
            "product__{}__meta_title".format(lang),
            "product__{}__meta_description".format(lang),
            "product__{}__description_short".format(lang),
            "product__{}__description_long".format(lang),
            "weight",
            "product__weight",
            "code",
            family_name="product__family__{}__name".format(lang),
            family_slug="product__family__{}__slug".format(lang),
            family_description="product__family__{}__description".format(lang),
            category_slug="product__category__{}__slug".format(lang),
            category_name="product__category__{}__name".format(lang),
            category_description="product__category__{}__description".format(lang),
            subcategory_slug="product__subcategory__{}__slug".format(lang),
            subcategory_name="product__subcategory__{}__name".format(lang),
            subcategory_description="product__subcategory__{}__description".format(lang),
            family_image="product__family__image",
            family_icon="product__family__icon",
            category_image="product__category__image",
            subcategory_image="product__subcategory__image",
            tax="product__tax__tax",
            meta_title="{}__meta_title".format(lang),
            meta_description="{}__meta_description".format(lang),
            meta_keywords="{}__meta_keywords".format(lang),
            description_short="{}__description_short".format(lang),
            description_long="{}__description_long".format(lang),
            description_sample="{}__description_sample".format(lang),
            name="{}__name".format(lang),
            slug="{}__slug".format(lang),
            product_meta_title="product__{}__meta_title".format(lang),
            product_meta_description="product__{}__meta_description".format(lang),
            product_description_short="product__{}__description_short".format(lang),
            product_description_long="product__{}__description_long".format(lang),
            stars="reviews_value",
            pop_annotations=True
        )
        if onlypublic:
            product = product.exclude(product__model=False)
        product = product.first()

        if product:
            product_final = cls.objects.get(pk=product['pk'])
            prices = product_final.calculate_price()
            product['price'] = prices['price_total']

        return product

    def get_value_product_unique(self, pos):
        """
        Return all products unique relationship with POS's Storage (only salable zones)
        """
        qs = ProductUnique.objects.filter(
            box__box_structure__zone__storage__in=pos.storage_stock.filter(storage_zones__salable=True),
            product_final=self
        )
        return qs


# imagenes de productos
class ProductFinalImage(CodenerixModel, GenImageFile):
    product_final = models.ForeignKey(ProductFinal, on_delete=models.CASCADE, related_name='productfinals_image', verbose_name=_("Product Final"))
    order = models.SmallIntegerField(_("Order"), blank=True, null=True)
    public = models.BooleanField(_("Public"), blank=True, null=False, default=True)
    principal = models.BooleanField(_("Principal"), blank=False, null=False, default=False)
    flagship_product = models.BooleanField(_("Flagship product"), default=False)
    outstanding = models.BooleanField(_("Outstanding"), default=False)

    def __unicode__(self):
        return u"{} ({})".format(smart_text(self.product_final), smart_text(self.order))

    def __str__(self):
        return self.__unicode__()

    def __fields__(self, info):
        fields = []
        fields.append(('product_final', _("Product")))
        fields.append(('order', _("Order")))
        fields.append(('public', _("Public")))
        fields.append(('principal', _("Principal")))
        return fields

    # Save necesita un check que indique si debe comprobar o no los productos destacados y productos estrella.
    @transaction.atomic
    def save(self, *args, **kwards):
        if self.principal:
            ProductFinalImage.objects.filter(product_final=self.product_final).exclude(pk=self.pk).update(principal=False)
        elif not ProductFinalImage.objects.exclude(pk=self.pk).filter(principal=True).exists():
            self.principal = True

        # Si no hay ninguna imagen con el check de producto estrella, marco esta.
        if self.flagship_product:
            ProductFinalImage.objects.filter(product_final=self.product_final, flagship_product=True).exclude(pk=self.pk).update(flagship_product=False)
        elif not ProductFinalImage.objects.filter(product_final=self.product_final, flagship_product=True).exclude(pk=self.pk).exists():
            self.flagship_product = True

        # Producto destacado
        if self.outstanding:
            ProductFinalImage.objects.filter(product_final=self.product_final, outstanding=True).exclude(pk=self.pk).update(outstanding=False)
        elif not ProductFinalImage.objects.filter(product_final=self.product_final, outstanding=True).exclude(pk=self.pk).exists():
            self.outstanding = True

        return super(ProductFinalImage, self).save(*args, **kwards)


# producto final (1 producto muchos atributos) (pulgadas, RAM)
class ProductFinalAttribute(CodenerixModel):
    """
    el stock se relaciona con esta clase
    definición de productos individuales
    """
    product = models.ForeignKey(ProductFinal, on_delete=models.CASCADE, blank=False, null=False, related_name='products_final_attr', verbose_name=_('Product'))
    attribute = models.ForeignKey(Attribute, on_delete=models.CASCADE, blank=False, null=True, related_name='products_final_attr', verbose_name=_('Attributes'))

    value = models.CharField(_("Value"), max_length=80)

    def __unicode__(self, show_attribute=True):
        value = ''
        if self.attribute.type_value == TYPE_VALUE_BOOLEAN:
            value = bool(self.value) and _('True') or _('False')
        elif self.attribute.type_value == TYPE_VALUE_FREE:
            value = self.value
        elif self.attribute.type_value == TYPE_VALUE_LIST:
            lang = get_language_database()
            field = '{}__description'.format(lang)
            ov = OptionValueAttribute.objects.filter(
                group=self.attribute.list_value,
                pk=int(self.value)
            ).values(
                field
            ).first()
            if ov:
                value = ov[field]
        if show_attribute:
            return u"{}: {}".format(smart_text(self.attribute), smart_text(value))
        else:
            return u"{}".format(smart_text(value))

    def __str__(self):
        return self.__unicode__()

    def get_value_attribute(self):
        return self.__unicode__(show_attribute=False)

    def __fields__(self, info):
        fields = []
        fields.append(('product', _("Product")))
        fields.append(('attribute', _("Attribute")))
        fields.append(('get_value_attribute', _("Value")))
        return fields

    @staticmethod
    def validate(pk, product, attribute):
        if pk is None and ProductFinalAttribute.objects.filter(product__pk=product, attribute__pk=attribute).exists():
            msg = _('A product final can not have the same attribute')
        elif ProductFinalAttribute.objects.filter(product__pk=product, attribute__pk=attribute).exclude(pk=pk).exists():
            msg = _('A product final can not have the same attribute')
        else:
            msg = None

        return msg

    def save(self, *args, **kwargs):
        msg = ProductFinalAttribute.validate(self.pk, self.product.pk, self.attribute.pk)
        if msg:
            raise ProductFinalAttributeOnlyOne(msg)
        return super(ProductFinalAttribute, self).save(*args, **kwargs)


# valor de las caracteristicas del producto (talla, color)
class ProductFeature(CodenerixModel):
    product = models.ForeignKey(Product, on_delete=models.CASCADE, blank=False, null=False, verbose_name=_('Product'), related_name='product_features')
    feature = models.ForeignKey(Feature, on_delete=models.CASCADE, blank=False, null=False, verbose_name=_('Feature'), related_name='product_features')
    value = models.CharField(_("Value"), max_length=80)

    def __unicode__(self):
        return u"{}".format(smart_text(self.product))

    def __str__(self):
        return self.__unicode__()

    def __fields__(self, info):
        fields = []
        fields.append(('product', _("Product")))
        fields.append(('feature', _("Feature")))
        fields.append(('value', _("Value")))
        return fields


# valor de las caracteristicas especiales del producto final (imei, fecha caducidad)
class ProductUnique(CodenerixModel):
    product_final = models.ForeignKey(ProductFinal, on_delete=models.CASCADE, blank=False, null=False, related_name='products_unique', verbose_name=_('Product final'))
    box = models.ForeignKey(StorageBox, on_delete=models.CASCADE, blank=False, null=False, related_name='products_unique', verbose_name=_('Box'))
    value = models.CharField(_("Value"), max_length=80, null=True, blank=True)
    caducity = models.DateField(_("Caducity"), blank=True, null=True, default=None)
    stock_original = models.FloatField(_("Stock original"), null=False, blank=False, default=0)
    stock_real = models.FloatField(_("Stock real"), null=False, blank=False, default=0, editable=False)
    stock_locked = models.FloatField(_("Stock locked"), null=False, blank=False, default=0, editable=False)

    def __str__(self):
        if self.value:
            result = self.value
        else:
            result = self.product_final
        return u"{} ({})".format(smart_text(result), self.box)

    def __fields__(self, info):
        fields = []
        fields.append(('product_final', _("Product")))
        fields.append(('product_final__product__feature_special', _("Feature special")))
        fields.append(('value', _("Value")))
        fields.append(('box', _("Box")))
        fields.append(('stock_original', _("Stock original")))
        fields.append(('stock_real', _("Stock real")))
        fields.append(('stock_locked', _("Stock locked")))
        return fields

    def locked_stock(self, quantity):
        self.stock_locked += quantity
        self.save()

    @transaction.atomic
    def duplicate(self, quantity, locked=False):
        # Do we have enought stock?
        if self.stock_real > quantity:
            if locked:
                # Do we have enought locked stock?
                if self.stock_lock>=quantity:
                    newlock = quantity
                else:
                    raise IOError("Not enought locked products to split")
            else:
                # Find available stock here
                available = self.stock_real-self.stock_lock
                # Do you have enought free stock?
                if available>=quantity:
                    newlock = 0
                else:
                    raise IOError("Not enought free products to split")

            # Make a copy from the actual ProductUnique
            new_line = copy.copy(self)
            new_line.pk = None
            # Complete stocks as calculated
            new_line.stock_original = quantity
            new_line.stock_real = quantity
            new_line.stock_lock = newlock
            new_line.save()
            self.stock_original -= quantity
            self.stock_real -= quantity
            self.stock_lock -= newlock
            self.save()

            # Check PurchaseAlbaran and link them to the new product
            for pal in self.line_albaran_purchases.all():
                pal.product_unique.add(new_line)
                pal.save()

            # Return new ProductUnique
            return new_line
        elif self.stock_real == quantity:
            raise IOError("No need to split, you are taking all units from here")
        else:
            raise IOError("Not enought products to split")

    def save(self, *args, **kwargs):
        product_final = ProductFinal.objects.filter(pk=self.product_final_id).first()
        # se comprueba que no se repite el valor de las caracteristicas especiales de los productos finales cuando sean unicas
        if product_final:
            if product_final.product.feature_special and product_final.product.feature_special.unique:
                if ProductUnique.objects.filter(
                    value=self.value,
                    product_final__product=product_final.product
                ).exclude(
                    pk=self.pk
                ).exists():
                    raise ProductUniqueAlreadyExists(_('Ya existe un producto unico con el valor de la caracteristicas especial'))
                elif self.stock_original > 1:
                    raise ProductUniqueQuantityExceeded(_('Este producto unico solo se puede cargar en cantidades de uno, porque la caracteristica especian indica que debe el valor no se puede repetir'))

        else:
            raise ProductUniqueNotProductFinal(_("Product final not seleted"))

        if self.pk is None:
            self.stock_real = self.stock_original
            self.stock_locked = 0

        return super(ProductUnique, self).save(*args, **kwargs)


# producto estrella (solo un registro publico)
class FlagshipProduct(CustomQueryMixin, CodenerixModel, GenImageFile):
    product_final = models.ForeignKey(ProductFinal, on_delete=models.CASCADE, blank=False, null=False, related_name='flagship_products', verbose_name=_('Flagship product'))
    public = models.BooleanField(_("Public"), blank=True, null=False, default=True)
    view_video = models.BooleanField(_("View video"), blank=True, null=False, default=False)
    orientazion = models.CharField(_("Orientazion"), max_length=2, choices=TYPE_ORIENTAZION, blank=False, null=False, default='R')

    def __unicode__(self):
        return u"{}".format(smart_text(self.product_final))

    def __str__(self):
        return self.__unicode__()

    def __fields__(self, info):
        fields = []
        fields.append(('product_final', _("Product")))
        fields.append(('public', _("Public")))
        fields.append(('view_video', _("View Video")))
        fields.append(('orientazion', _("Orientazion")))
        return fields

    def save(self, *args, **kwards):
        if self.public:
            if self.pk:
                FlagshipProduct.objects.exclude(pk=self.pk).update(public=False)
            else:
                FlagshipProduct.objects.all().update(public=False)
        return super(FlagshipProduct, self).save(*args, **kwards)

    @classmethod
    def get_flagship(cls, lang, apply_surcharge=False):
        flagship = cls.query_or(
            Q(public=True),
            "image",
            "view_video",
            "product_final__pk",
            "product_final__price",
            "product_final__product__tax__tax",
            "product_final__product__url_video",
            "product_final__{}__slug".format(lang),
            "orientazion",
            "{}__title".format(lang),
            "{}__description".format(lang),
            pk="product_final__pk",
            price="product_final__price",
            tax="product_final__product__tax__tax",
            slug="product_final__{}__slug".format(lang),
            title="{}__title".format(lang),
            description="{}__description".format(lang),
            pop_annotations=True
        ).first()

        if flagship:
            product_final = ProductFinal.objects.get(pk=flagship['pk'])
            prices = product_final.calculate_price(apply_surcharge)
            flagship['price'] = prices['price_total']

        return flagship


class ProductFinalOption(CodenerixModel):
    product_final = models.ForeignKey(ProductFinal, on_delete=models.CASCADE, related_name='productfinals_option', verbose_name=_("Product Final"))
    products_pack = models.ManyToManyField(ProductFinal, related_name='productfinals_optionpack', symmetrical=False, blank=False)
    active = models.BooleanField(_("Active"), blank=False, null=False, default=True)
    order = models.SmallIntegerField(_("Order"), blank=True, null=True)

    def __unicode__(self):
        lang = get_language_database()
        return u"{}".format(smart_text(getattr(self, lang).name))

    def __str__(self):
        return self.__unicode__()

    def __fields__(self, info):
        lang = get_language_database()
        fields = []
        fields.append(('product_final', _("Product final")))
        fields.append(('{}__name'.format(lang), _("Option")))
        fields.append(('order', _("Order")))
        fields.append(('products_pack', _("Products options")))
        fields.append(('active', _("Active")))
        return fields


MODELS_SLUG = [
    ("family", "Family"),
    ("category", "Category"),
    ("subcategory", "Subcategory"),
    ("product_final_option", "ProductFinalOption"),
]

for info in MODELS_SLUG:
    field = info[0]
    model = info[1]
    for lang_code in settings.LANGUAGES_DATABASES:
        query = "class {}Text{}(GenTextSlug):\n".format(model, lang_code)
        query += "  {} = models.OneToOneField({}, on_delete=models.CASCADE, blank=False, null=False, related_name='{}')\n".format(field, model, lang_code.lower())
        exec(query)

MODELS = [
    ("feature", "Feature"),
    ("attribute", "Attribute"),
    ("feature_special", "FeatureSpecial"),
    ("product_image", "ProductImage"),        # para los alt y los titles de las imagenes
    ("product_final_image", "ProductFinalImage"),
    ("product_document", "ProductDocument"),  # para los titles de los enlaces
    ("option_value", "OptionValueFeature"),
    ("option_value", "OptionValueAttribute"),
    ("option_value", "OptionValueFeatureSpecial"),
]

for info in MODELS:
    field = info[0]
    model = info[1]
    for lang_code in settings.LANGUAGES_DATABASES:
        query = "class {}Text{}(GenText):\n".format(model, lang_code)
        query += "  {} = models.OneToOneField({}, on_delete=models.CASCADE, blank=False, null=False, related_name='{}')\n".format(field, model, lang_code.lower())
        exec(query)


MODELS_PRODUCTS = [
    ('product', 'ProductText', 'Product'),
]

for info in MODELS_PRODUCTS:
    field = info[0]
    model_source = info[1]
    model_relate = info[2]
    for lang_code in settings.LANGUAGES_DATABASES:
        query = "class {}Text{}(GenProductText):\n".format(model_source, lang_code)
        query += "  {} = models.OneToOneField({}, on_delete=models.CASCADE, blank=False, null=False, related_name='{}')\n".format(field, model_relate, lang_code.lower())
        exec(query)

MODELS_BRANDS = [
    ('brand', 'Brand', 'Brand'),
]

for info in MODELS_BRANDS:
    field = info[0]
    model_source = info[1]
    model_relate = info[2]
    for lang_code in settings.LANGUAGES_DATABASES:
        query = "class {}Text{}(GenBrandText):\n".format(model_source, lang_code)
        query += "  {} = models.OneToOneField({}, on_delete=models.CASCADE, blank=False, null=False, related_name='{}')\n".format(field, model_relate, lang_code.lower())
        exec(query)

MODELS_PRODUCTS_FINAL = [
    ('product', 'ProductFinal', 'ProductFinal'),
]

for info in MODELS_PRODUCTS_FINAL:
    field = info[0]
    model_source = info[1]
    model_relate = info[2]
    for lang_code in settings.LANGUAGES_DATABASES:
        query = "class {}Text{}(GenProductFinalText):\n".format(model_source, lang_code)
        query += "  {} = models.OneToOneField({}, on_delete=models.CASCADE, blank=False, null=False, related_name='{}')\n".format(field, model_relate, lang_code.lower())
        exec(query)

MODELS_SLIDERS = [
    ('product', 'FlagshipProduct'),
]

for info in MODELS_SLIDERS:
    field = info[0]
    model = info[1]
    for lang_code in settings.LANGUAGES_DATABASES:
        query = "class {}Text{}(GenTextTitle):\n".format(model, lang_code)
        query += "  {} = models.OneToOneField({}, on_delete=models.CASCADE, blank=False, null=False, related_name='{}')\n".format(field, model, lang_code.lower())
        exec(query)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-04-26 12:20
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_products', '0011_auto_20180202_0826'),
    ]

    operations = [
        migrations.AddField(
            model_name='productunique',
            name='caducity',
            field=models.DateField(blank=True, default=None, null=True, verbose_name='Caducity'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-01-18 10:26
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_products', '0005_auto_20180118_0739'),
    ]

    operations = [
        migrations.AlterField(
            model_name='brandtexten',
            name='name',
            field=models.CharField(default='test', max_length=250, verbose_name='Name'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='brandtextes',
            name='name',
            field=models.CharField(default='test', max_length=250, verbose_name='Name'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='productfinaltexten',
            name='name',
            field=models.CharField(default='test', max_length=250, verbose_name='Name'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='productfinaltextes',
            name='name',
            field=models.CharField(default='test', max_length=250, verbose_name='Name'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='producttexttexten',
            name='name',
            field=models.CharField(default='test', max_length=250, verbose_name='Name'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='producttexttextes',
            name='name',
            field=models.CharField(default='test', max_length=250, verbose_name='Name'),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-02-02 06:26
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_products', '0009_auto_20180201_1137'),
        ('codenerix_products', '0007_product_caducable'),
    ]

    operations = [
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-01-17 15:55
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_products', '0002_auto_20180117_1628'),
    ]

    operations = [
        migrations.AlterField(
            model_name='product',
            name='packing_cost',
            field=models.DecimalField(decimal_places=2, default=0, max_digits=10, verbose_name='Packing cost'),
        ),
        migrations.AlterField(
            model_name='productfinal',
            name='packing_cost',
            field=models.DecimalField(blank=True, decimal_places=2, help_text='If it is empty, packing cost is equal to packing cost of product', max_digits=10, null=True, verbose_name='Packing cost'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-02-01 10:37
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_products', '0008_auto_20180126_1711'),
    ]

    operations = [
        migrations.AlterField(
            model_name='productunique',
            name='box',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='products_unique', to='codenerix_storages.StorageBox', verbose_name='Box'),
        ),
        migrations.AlterField(
            model_name='productunique',
            name='stock_original',
            field=models.FloatField(default=0, verbose_name='Stock original'),
        ),
    ]



# -*- coding: utf-8 -*-
#
# django-codenerix-products
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.conf.urls import url
from .views import FeatureList, AttributeList, FeatureSpecialList, FamilyList, CategoryList, SubcategoryList, ProductList, ProductRelationSoldList, ProductImageList, ProductDocumentList, ProductFinalList, ProductFeatureList, ProductUniqueList
from .views import FeatureCreate, AttributeCreate, FeatureSpecialCreate, FamilyCreate, CategoryCreate, SubcategoryCreate, ProductCreate, ProductRelationSoldCreate, ProductFinalCreate
from .views import FeatureCreateModal, AttributeCreateModal, FeatureSpecialCreateModal, FamilyCreateModal, CategoryCreateModal, SubcategoryCreateModal, SubcategoryCreateModalAll, ProductCreateModal, ProductRelationSoldCreateModal, ProductImageCreateModal, ProductDocumentCreateModal, ProductFinalCreateModal, ProductFeatureCreateModal, ProductUniqueCreateModal
from .views import FeatureUpdate, AttributeUpdate, FeatureSpecialUpdate, FamilyUpdate, CategoryUpdate, SubcategoryUpdate, ProductUpdate, ProductRelationSoldUpdate, ProductImageUpdate, ProductDocumentUpdate, ProductFinalUpdate, ProductFeatureUpdate, ProductUniqueUpdate
from .views import FeatureUpdateModal, AttributeUpdateModal, FeatureSpecialUpdateModal, FamilyUpdateModal, CategoryUpdateModal, SubcategoryUpdateModal, ProductUpdateModal, ProductRelationSoldUpdateModal, ProductImageUpdateModal, ProductDocumentUpdateModal, ProductFinalUpdateModal, ProductFeatureUpdateModal, ProductUniqueUpdateModal
from .views import FeatureDelete, AttributeDelete, FeatureSpecialDelete, FamilyDelete, CategoryDelete, SubcategoryDelete, ProductDelete, ProductRelationSoldDelete, ProductImageDelete, ProductDocumentDelete, ProductFinalDelete, ProductFeatureDelete, ProductUniqueDelete
from .views import ProductDetails, ProductFeatureSubList, ProductFeatureDetailsModal
from .views import ProductDocumentSubList, ProductDocumentDetailsModal
from .views import ProductFinalDetails, CategoryDetails, ProductFinalFullinfo
# ProductFinalImageCreate,
from .views import ProductFinalImageList, ProductFinalImageCreateModal, ProductFinalImageUpdate, ProductFinalImageUpdateModal, ProductFinalImageDelete, ProductFinalImageSubList, ProductFinalImageDetails, ProductFinalImageDetailsModal
from .views import ProductFinalAttributeList, ProductFinalAttributeSubList, ProductFinalAttributeCreateModal, ProductFinalAttributeDetailsModal, ProductFinalAttributeUpdateModal, ProductFinalAttributeDelete
from .views import ProductImageSubList, ProductImageDetailsModal
from .views import ProductUniqueSubList, ProductUniqueDetailsModal, ProductUniqueCodeForeign
from .views import CategoryForeign, SubcategoryForeign, FeatureForeign
from .views import ProductFinalRelatedSubList, ProductFinalRelatedSubUpdateModal, ProductFinalRelatedSubDelete
from .views import ProductFinalAccesorySubList, ProductFinalAccesorySubUpdateModal, ProductFinalAccesorySubDelete
from .views import ProductForeign
from .views import AttributeForeign, FeatureSpecialForeign
from .views import TypeTaxList, TypeTaxCreate, TypeTaxCreateModal, TypeTaxUpdate, TypeTaxUpdateModal, TypeTaxDelete
from .views import SubcategorySubList, SubcategoryDetailModal
from .views import BrandList, BrandCreate, BrandCreateModal, BrandUpdate, BrandUpdateModal, BrandDelete
from .views import FlagshipProductList, FlagshipProductCreate, FlagshipProductCreateModal, FlagshipProductUpdate, FlagshipProductUpdateModal, FlagshipProductDelete
from .views import CategorySubListPro, CategoryDetailModalPro, CategoryUpdateModalPro
from .views import ListProducts, ListProductsBase, TypeTaxDetails
from .views import ProductFinalSubList, ProductFinalDetailsModal
from .views import GroupValueFeatureList, GroupValueFeatureCreate, GroupValueFeatureCreateModal, GroupValueFeatureDetails, GroupValueFeatureUpdate, GroupValueFeatureUpdateModal, GroupValueFeatureDelete
from .views import GroupValueAttributeList, GroupValueAttributeCreate, GroupValueAttributeCreateModal, GroupValueAttributeDetails, GroupValueAttributeUpdate, GroupValueAttributeUpdateModal, GroupValueAttributeDelete
from .views import GroupValueFeatureSpecialList, GroupValueFeatureSpecialCreate, GroupValueFeatureSpecialCreateModal, GroupValueFeatureSpecialDetails, GroupValueFeatureSpecialUpdate, GroupValueFeatureSpecialUpdateModal, GroupValueFeatureSpecialDelete
from .views import OptionValueFeatureList, OptionValueFeatureUpdate, OptionValueFeatureUpdateModal, OptionValueFeatureDelete, OptionValueFeatureSubList, OptionValueFeatureSubListModal, OptionValueFeatureCreateModal, OptionValueFeatureDetailsModal, OptionValueFeatureForeign
from .views import OptionValueAttributeList, OptionValueAttributeUpdate, OptionValueAttributeUpdateModal, OptionValueAttributeDelete, OptionValueAttributeSubList, OptionValueAttributeSubListModal, OptionValueAttributeCreateModal, OptionValueAttributeDetailsModal, OptionValueAttributeForeign
from .views import OptionValueFeatureSpecialList, OptionValueFeatureSpecialUpdate, OptionValueFeatureSpecialUpdateModal, OptionValueFeatureSpecialDelete, OptionValueFeatureSpecialSubList, OptionValueFeatureSpecialSubListModal, OptionValueFeatureSpecialCreateModal, OptionValueFeatureSpecialDetailsModal, OptionValueFeatureSpecialForeign
from .views import ProductFinalOptionList, ProductFinalOptionCreate, ProductFinalOptionCreateModal, ProductFinalOptionUpdate, ProductFinalOptionUpdateModal, ProductFinalOptionDelete, ProductFinalOptionSubList, ProductFinalOptionDetails, ProductFinalOptionDetailModal
from .views import ProductFinalEAN13Foreign
from .views import ProductFinalForeignSales, ProductFinalForeignPackSales, ProductFinalForeignAllSales
from .views import ProductFinalForeignPurchases, ProductFinalForeignPackPurchases, ProductFinalForeignAllPurchases
from .views import TypeTaxForeign
from .views import ProductCreateCustom
from .views import ProductUniqueForeign


urlpatterns = [
    url(r'^typetaxs$', TypeTaxList.as_view(), name='CDNX_products_typetaxs_list'),
    url(r'^typetaxs/add$', TypeTaxCreate.as_view(), name='CDNX_products_typetaxs_add'),
    url(r'^typetaxs/addmodal$', TypeTaxCreateModal.as_view(), name='CDNX_products_typetaxs_addmodal'),
    url(r'^typetaxs/(?P<pk>\w+)$', TypeTaxDetails.as_view(), name='CDNX_products_categorys_details'),
    url(r'^typetaxs/(?P<pk>\w+)/edit$', TypeTaxUpdate.as_view(), name='CDNX_products_typetaxs_edit'),
    url(r'^typetaxs/(?P<pk>\w+)/editmodal$', TypeTaxUpdateModal.as_view(), name='CDNX_products_typetaxs_editmodal'),
    url(r'^typetaxs/(?P<pk>\w+)/delete$', TypeTaxDelete.as_view(), name='CDNX_products_typetaxs_delete'),
    url(r'^typetaxs/foreign/(?P<search>[\w\W]+|\*)$', TypeTaxForeign.as_view(), name='CDNX_products_typetaxs_foreign'),

    url(r'^features$', FeatureList.as_view(), name='CDNX_products_features_list'),
    url(r'^features/add$', FeatureCreate.as_view(), name='CDNX_products_features_add'),
    url(r'^features/addmodal$', FeatureCreateModal.as_view(), name='CDNX_products_features_addmodal'),
    url(r'^features/(?P<pk>\w+)/edit$', FeatureUpdate.as_view(), name='CDNX_products_features_edit'),
    url(r'^features/(?P<pk>\w+)/editmodal$', FeatureUpdateModal.as_view(), name='CDNX_products_features_editmodal'),
    url(r'^features/(?P<pk>\w+)/delete$', FeatureDelete.as_view(), name='CDNX_products_features_delete'),
    url(r'^features/foreign/(?P<search>[\w\W]+|\*)$', FeatureForeign.as_view(), name='CDNX_products_features_foreign'),

    url(r'^attributes$', AttributeList.as_view(), name='CDNX_products_attributes_list'),
    url(r'^attributes/add$', AttributeCreate.as_view(), name='CDNX_products_attributes_add'),
    url(r'^attributes/addmodal$', AttributeCreateModal.as_view(), name='CDNX_products_attributes_addmodal'),
    url(r'^attributes/(?P<pk>\w+)/edit$', AttributeUpdate.as_view(), name='CDNX_products_attributes_edit'),
    url(r'^attributes/(?P<pk>\w+)/editmodal$', AttributeUpdateModal.as_view(), name='CDNX_products_attributes_editmodal'),
    url(r'^attributes/(?P<pk>\w+)/delete$', AttributeDelete.as_view(), name='CDNX_products_attributes_delete'),
    url(r'^attributes/foreign/(?P<search>[\w\W]+|\*)$', AttributeForeign.as_view(), name='CDNX_products_attributes_foreign'),

    url(r'^featurespecials$', FeatureSpecialList.as_view(), name='CDNX_products_featurespecials_list'),
    url(r'^featurespecials/add$', FeatureSpecialCreate.as_view(), name='CDNX_products_featurespecials_add'),
    url(r'^featurespecials/addmodal$', FeatureSpecialCreateModal.as_view(), name='CDNX_products_featurespecials_addmodal'),
    url(r'^featurespecials/(?P<pk>\w+)/edit$', FeatureSpecialUpdate.as_view(), name='CDNX_products_featurespecials_edit'),
    url(r'^featurespecials/(?P<pk>\w+)/editmodal$', FeatureSpecialUpdateModal.as_view(), name='CDNX_products_featurespecials_editmodal'),
    url(r'^featurespecials/(?P<pk>\w+)/delete$', FeatureSpecialDelete.as_view(), name='CDNX_products_featurespecials_delete'),
    url(r'^featurespecials/foreign/(?P<search>[\w\W]+|\*)$', FeatureSpecialForeign.as_view(), name='CDNX_products_featurespecials_foreign'),

    url(r'^familys$', FamilyList.as_view(), name='CDNX_products_familys_list'),
    url(r'^familys/add$', FamilyCreate.as_view(), name='CDNX_products_familys_add'),
    url(r'^familys/addmodal$', FamilyCreateModal.as_view(), name='CDNX_products_familys_addmodal'),
    url(r'^familys/(?P<pk>\w+)/edit$', FamilyUpdate.as_view(), name='CDNX_products_familys_edit'),
    url(r'^familys/(?P<pk>\w+)/editmodal$', FamilyUpdateModal.as_view(), name='CDNX_products_familys_editmodal'),
    url(r'^familys/(?P<pk>\w+)/delete$', FamilyDelete.as_view(), name='CDNX_products_familys_delete'),

    url(r'^brands$', BrandList.as_view(), name='CDNX_products_brand_list'),
    url(r'^brands/add$', BrandCreate.as_view(), name='CDNX_products_brand_add'),
    url(r'^brands/addmodal$', BrandCreateModal.as_view(), name='CDNX_products_brand_addmodal'),
    url(r'^brands/(?P<pk>\w+)/edit$', BrandUpdate.as_view(), name='CDNX_products_brand_edit'),
    url(r'^brands/(?P<pk>\w+)/editmodal$', BrandUpdateModal.as_view(), name='CDNX_products_brand_editmodal'),
    url(r'^brands/(?P<pk>\w+)/delete$', BrandDelete.as_view(), name='CDNX_products_brand_delete'),

    url(r'^categorys$', CategoryList.as_view(), name='CDNX_products_categorys_list'),
    url(r'^categorys/add$', CategoryCreate.as_view(), name='CDNX_products_categorys_add'),
    url(r'^categorys/addmodal$', CategoryCreateModal.as_view(), name='CDNX_products_categorys_addmodal'),
    url(r'^categorys/(?P<pk>\w+)$', CategoryDetails.as_view(), name='CDNX_products_categorys_details'),
    url(r'^categorys/(?P<pk>\w+)/edit$', CategoryUpdate.as_view(), name='CDNX_products_categorys_edit'),
    url(r'^categorys/(?P<pk>\w+)/editmodal$', CategoryUpdateModal.as_view(), name='CDNX_products_categorys_editmodal'),
    url(r'^categorys/(?P<pk>\w+)/delete$', CategoryDelete.as_view(), name='CDNX_products_categorys_delete'),
    url(r'^categorys/foreign/(?P<search>[\w\W]+|\*)$', CategoryForeign.as_view(), name='CDNX_products_categorys_foreign'),

    url(r'^categorys/(?P<pk>\w+)/sublistpro$', CategorySubListPro.as_view(), name='CDNX_products_categoryrelateds_sublist'),
    url(r'^categorys/(?P<cpk>\w+)/sublistpro/(?P<pk>\w+)$', CategoryDetailModalPro.as_view(), name='CDNX_products_category_sublist_details'),
    url(r'^categorys/(?P<cpk>\w+)/sublistpro/(?P<pk>\w+)/edit$', CategoryUpdateModalPro.as_view(), name='CDNX_products_category_sublist_edit'),
    url(r'^categorys/(?P<cpk>\w+)/sublistpro/(?P<pk>\w+)/editmodal$', CategoryUpdateModalPro.as_view(), name='CDNX_products_category_sublist_editmodal'),
    url(r'^categorys/(?P<cpk>\w+)/sublistpro/(?P<pk>\w+)/delete$', CategoryDelete.as_view(), name='CDNX_products_category_sublist_delete'),

    url(r'^subcategorys$', SubcategoryList.as_view(), name='CDNX_products_subcategorys_list'),
    url(r'^subcategorys/add$', SubcategoryCreate.as_view(), name='CDNX_products_subcategorys_add'),
    url(r'^subcategorys/addmodal$', SubcategoryCreateModalAll.as_view(), name='CDNX_products_subcategorys_addmodal'),
    url(r'^subcategorys/(?P<pk>\w+)/edit$', SubcategoryUpdate.as_view(), name='CDNX_products_subcategorys_edit'),
    url(r'^subcategorys/(?P<pk>\w+)/editmodal$', SubcategoryUpdateModal.as_view(), name='CDNX_products_subcategorys_editmodal'),
    url(r'^subcategorys/(?P<pk>\w+)/delete$', SubcategoryDelete.as_view(), name='CDNX_products_subcategorys_delete'),
    url(r'^subcategorys/foreign/(?P<search>[\w\W]+|\*)$', SubcategoryForeign.as_view(), name='CDNX_products_subcategorys_foreign'),

    url(r'^subcategorys/(?P<pk>\w+)/sublist$', SubcategorySubList.as_view(), name='CDNX_products_subcategory_sublist'),
    url(r'^subcategorys/(?P<cpk>\w+)/sublist/add$', SubcategoryCreateModal.as_view(), name='CDNX_products_subcategory_sublist_add'),
    url(r'^subcategorys/(?P<cpk>\w+)/sublist/addmodal$', SubcategoryCreateModal.as_view(), name='CDNX_products_subcategory_sublist_addmodal'),
    url(r'^subcategorys/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', SubcategoryDetailModal.as_view(), name='CDNX_products_subcategory_sublist_details'),
    url(r'^subcategorys/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', SubcategoryUpdateModal.as_view(), name='CDNX_products_subcategory_sublist_edit'),
    url(r'^subcategorys/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', SubcategoryUpdateModal.as_view(), name='CDNX_products_subcategory_sublist_editmodal'),
    url(r'^subcategorys/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', SubcategoryDelete.as_view(), name='CDNX_products_subcategory_sublist_delete'),

    url(r'^products$', ProductList.as_view(), name='CDNX_products_products_list'),
    url(r'^products/add$', ProductCreate.as_view(), name='CDNX_products_products_add'),
    url(r'^products/custom/add$', ProductCreateCustom.as_view(), name='CDNX_products_products_addcustom'),
    url(r'^products/addmodal$', ProductCreateModal.as_view(), name='CDNX_products_products_addmodal'),
    url(r'^products/(?P<pk>\w+)$', ProductDetails.as_view(), name='CDNX_products_products_details'),
    url(r'^products/(?P<pk>\w+)/edit$', ProductUpdate.as_view(), name='CDNX_products_products_edit'),
    url(r'^products/(?P<pk>\w+)/editmodal$', ProductUpdateModal.as_view(), name='CDNX_products_products_editmodal'),
    url(r'^products/(?P<pk>\w+)/delete$', ProductDelete.as_view(), name='CDNX_products_products_delete'),
    url(r'^products/foreign/(?P<search>[\w\W]+|\*)$', ProductForeign.as_view(), name='CDNX_products_products_foreign'),

    url(r'^productrelationsolds$', ProductRelationSoldList.as_view(), name='CDNX_products_productrelationsolds_list'),
    url(r'^productrelationsolds/add$', ProductRelationSoldCreate.as_view(), name='CDNX_products_productrelationsolds_add'),
    url(r'^productrelationsolds/addmodal$', ProductRelationSoldCreateModal.as_view(), name='CDNX_products_productrelationsolds_addmodal'),
    url(r'^productrelationsolds/(?P<pk>\w+)/edit$', ProductRelationSoldUpdate.as_view(), name='CDNX_products_productrelationsolds_edit'),
    url(r'^productrelationsolds/(?P<pk>\w+)/editmodal$', ProductRelationSoldUpdateModal.as_view(), name='CDNX_products_productrelationsolds_editmodal'),
    url(r'^productrelationsolds/(?P<pk>\w+)/delete$', ProductRelationSoldDelete.as_view(), name='CDNX_products_productrelationsolds_delete'),

    url(r'^productimages$', ProductImageList.as_view(), name='CDNX_products_productimages_list'),
    url(r'^productimages/(?P<pk>\w+)/edit$', ProductImageUpdate.as_view(), name='CDNX_products_productimages_edit'),
    url(r'^productimages/(?P<pk>\w+)/editmodal$', ProductImageUpdateModal.as_view(), name='CDNX_products_productimages_editmodal'),
    url(r'^productimages/(?P<pk>\w+)/delete$', ProductImageDelete.as_view(), name='CDNX_products_productimages_delete'),
    url(r'^productimages/(?P<pk>\w+)/sublist$', ProductImageSubList.as_view(), name='CDNX_products_productimages_sublist'),
    url(r'^productimages/(?P<cpk>\w+)/sublist/add$', ProductImageCreateModal.as_view(), name='CDNX_products_productimages_sublist_add'),
    url(r'^productimages/(?P<cpk>\w+)/sublist/addmodal$', ProductImageCreateModal.as_view(), name='CDNX_products_productimages_sublist_addmodal'),
    url(r'^productimages/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', ProductImageDetailsModal.as_view(), name='CDNX_products_productimages_sublist_details'),
    url(r'^productimages/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', ProductImageUpdateModal.as_view(), name='CDNX_products_productimages_sublist_edit'),
    url(r'^productimages/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', ProductImageUpdateModal.as_view(), name='CDNX_products_productimages_sublist_editmodal'),
    url(r'^productimages/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', ProductImageDelete.as_view(), name='CDNX_products_productimages_sublist_delete'),

    url(r'^productdocuments$', ProductDocumentList.as_view(), name='CDNX_products_productdocuments_list'),
    url(r'^productdocuments/(?P<pk>\w+)/edit$', ProductDocumentUpdate.as_view(), name='CDNX_products_productdocuments_edit'),
    url(r'^productdocuments/(?P<pk>\w+)/editmodal$', ProductDocumentUpdateModal.as_view(), name='CDNX_products_productdocuments_editmodal'),
    url(r'^productdocuments/(?P<pk>\w+)/delete$', ProductDocumentDelete.as_view(), name='CDNX_products_productdocuments_delete'),
    url(r'^productdocuments/(?P<pk>\w+)/sublist$', ProductDocumentSubList.as_view(), name='CDNX_products_productdocuments_sublist'),
    url(r'^productdocuments/(?P<cpk>\w+)/sublist/add$', ProductDocumentCreateModal.as_view(), name='CDNX_products_productdocuments_sublist_add'),
    url(r'^productdocuments/(?P<cpk>\w+)/sublist/addmodal$', ProductDocumentCreateModal.as_view(), name='CDNX_products_productdocuments_sublist_addmodal'),
    url(r'^productdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', ProductDocumentDetailsModal.as_view(), name='CDNX_products_productdocuments_sublist_details'),
    url(r'^productdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', ProductDocumentUpdateModal.as_view(), name='CDNX_products_productdocuments_sublist_edit'),
    url(r'^productdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', ProductDocumentUpdateModal.as_view(), name='CDNX_products_productdocuments_sublist_editmodal'),
    url(r'^productdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', ProductDocumentDelete.as_view(), name='CDNX_products_productdocuments_sublist_delete'),

    url(r'^productfinals$', ProductFinalList.as_view(), name='CDNX_products_productfinals_list'),
    url(r'^productfinals/add$', ProductFinalCreate.as_view(), name='CDNX_products_productfinals_add'),
    url(r'^productfinals/custom/add$', ProductCreateCustom.as_view(), name='CDNX_products_productfinals_addcustom'),
    url(r'^productfinals/addmodal$', ProductFinalCreateModal.as_view(), name='CDNX_products_productfinals_addmodal'),
    url(r'^productfinals/(?P<pk>\w+)$', ProductFinalDetails.as_view(), name='CDNX_products_productfinals_details'),
    url(r'^productfinals/(?P<pk>\w+)/fullinfo$', ProductFinalFullinfo.as_view(), name='CDNX_products_productfinals_fullinfo'),
    url(r'^productfinals/(?P<pk>\w+)/edit$', ProductFinalUpdate.as_view(), name='CDNX_products_productfinals_edit'),
    url(r'^productfinals/(?P<pk>\w+)/editmodal$', ProductFinalUpdateModal.as_view(), name='CDNX_products_productfinals_editmodal'),
    url(r'^productfinals/(?P<pk>\w+)/delete$', ProductFinalDelete.as_view(), name='CDNX_products_productfinals_delete'),

    url(r'^productfinals/foreignean13/(?P<search>[\w\W]+|\*)$', ProductFinalEAN13Foreign.as_view(), name='CDNX_products_productfinalsean13_foreign'),

    url(r'^productfinals/foreignsales/(?P<search>[\w\W]+|\*)$', ProductFinalForeignSales.as_view(), name='CDNX_products_productfinals_foreign_sales'),
    url(r'^productfinals/foreignsalespack/(?P<search>[\w\W]+|\*)$', ProductFinalForeignPackSales.as_view(), name='CDNX_products_productfinals_foreign_sales_pack'),
    url(r'^productfinals/foreignsalesall/(?P<search>[\w\W]+|\*)$', ProductFinalForeignAllSales.as_view(), name='CDNX_products_productfinals_foreign_sales_all'),

    url(r'^productfinals/foreignpurchases/(?P<search>[\w\W]+|\*)$', ProductFinalForeignPurchases.as_view(), name='CDNX_products_productfinals_foreign_purchases'),
    url(r'^productfinals/foreignpurchasespack/(?P<search>[\w\W]+|\*)$', ProductFinalForeignPackPurchases.as_view(), name='CDNX_products_productfinals_foreign_purchases_pack'),
    url(r'^productfinals/foreignpurchasesall/(?P<search>[\w\W]+|\*)$', ProductFinalForeignAllPurchases.as_view(), name='CDNX_products_productfinals_foreign_purchases_all'),

    url(r'^productfinals/(?P<pk>\w+)/related$', ProductFinalRelatedSubList.as_view(), name='CDNX_products_productfinalrelateds_sublist'),
    url(r'^productfinals/(?P<pk>\w+)/related/add$', ProductFinalRelatedSubUpdateModal.as_view(), name='CDNX_products_productfinalrelateds_sublist_add'),
    url(r'^productfinals/(?P<pk>\w+)/related/addmodal$', ProductFinalRelatedSubUpdateModal.as_view(), name='CDNX_products_productfinalrelateds_sublist_addmodal'),
    url(r'^productfinals/(?P<tpk>\w+)/related/(?P<pk>\w+)/delete$', ProductFinalRelatedSubDelete.as_view(), name='CDNX_products_productfinalrelateds_sublist_delete'),

    url(r'^productfinals/(?P<pk>\w+)/accesory$', ProductFinalAccesorySubList.as_view(), name='CDNX_products_productfinalaccesory_sublist'),
    url(r'^productfinals/(?P<pk>\w+)/accesory/add$', ProductFinalAccesorySubUpdateModal.as_view(), name='CDNX_products_productfinalaccesory_sublist_add'),
    url(r'^productfinals/(?P<pk>\w+)/accesory/addmodal$', ProductFinalAccesorySubUpdateModal.as_view(), name='CDNX_products_productfinalaccesory_sublist_addmodal'),
    url(r'^productfinals/(?P<tpk>\w+)/accesory/(?P<pk>\w+)/delete$', ProductFinalAccesorySubDelete.as_view(), name='CDNX_products_productfinalaccesory_sublist_delete'),

    url(r'^productfinals/(?P<pk>\w+)/sublist$', ProductFinalSubList.as_view(), name='CDNX_products_productfinals_sublist'),
    url(r'^productfinals/(?P<cpk>\w+)/sublist/add$', ProductFinalCreateModal.as_view(), name='CDNX_products_productfinals_sublist_add'),
    url(r'^productfinals/(?P<cpk>\w+)/sublist/addmodal$', ProductFinalCreateModal.as_view(), name='CDNX_products_productfinals_sublist_addmodal'),
    url(r'^productfinals/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', ProductFinalDetailsModal.as_view(), name='CDNX_products_productfinals_sublist_details'),
    url(r'^productfinals/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', ProductFinalUpdateModal.as_view(), name='CDNX_products_productfinals_sublist_edit'),
    url(r'^productfinals/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', ProductFinalUpdateModal.as_view(), name='CDNX_products_productfinals_sublist_editmodal'),
    url(r'^productfinals/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', ProductFinalDelete.as_view(), name='CDNX_products_productfinals_sublist_delete'),


    url(r'^productfinalimages$', ProductFinalImageList.as_view(), name='CDNX_products_productfinalimages_list'),
    url(r'^productfinalimages/(?P<pk>\w+)$', ProductFinalImageDetails.as_view(), name='CDNX_products_productfinalimages_edit'),
    url(r'^productfinalimages/(?P<pk>\w+)/edit$', ProductFinalImageUpdate.as_view(), name='CDNX_products_productfinalimages_edit'),
    url(r'^productfinalimages/(?P<pk>\w+)/edit$', ProductFinalImageUpdate.as_view(), name='CDNX_products_productfinalimages_edit'),
    url(r'^productfinalimages/(?P<pk>\w+)/editmodal$', ProductFinalImageUpdateModal.as_view(), name='CDNX_products_productfinalimages_editmodal'),
    url(r'^productfinalimages/(?P<pk>\w+)/delete$', ProductFinalImageDelete.as_view(), name='CDNX_products_productfinalimages_delete'),
    url(r'^productfinalimages/(?P<pk>\w+)/sublist$', ProductFinalImageSubList.as_view(), name='CDNX_products_productfinalimages_sublist'),
    url(r'^productfinalimages/(?P<cpk>\w+)/sublist/add$', ProductFinalImageCreateModal.as_view(), name='CDNX_products_productfinalimages_sublist_add'),
    url(r'^productfinalimages/(?P<cpk>\w+)/sublist/addmodal$', ProductFinalImageCreateModal.as_view(), name='CDNX_products_productfinalimages_sublist_addmodal'),
    url(r'^productfinalimages/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', ProductFinalImageDetailsModal.as_view(), name='CDNX_products_productfinalimages_sublist_details'),
    url(r'^productfinalimages/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', ProductFinalImageUpdateModal.as_view(), name='CDNX_products_productfinalimages_sublist_edit'),
    url(r'^productfinalimages/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', ProductFinalImageUpdateModal.as_view(), name='CDNX_products_productfinalimages_sublist_editmodal'),
    url(r'^productfinalimages/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', ProductFinalImageDelete.as_view(), name='CDNX_products_productfinalimages_sublist_delete'),

    url(r'^productfinalattributes$', ProductFinalAttributeList.as_view(), name='CDNX_products_productfinalattributes_list'),
    url(r'^productfinalattributes/(?P<pk>\w+)/editmodal$', ProductFinalAttributeUpdateModal.as_view(), name='CDNX_products_productfinalattributes_editmodal'),
    url(r'^productfinalattributes/(?P<pk>\w+)/delete$', ProductFinalAttributeDelete.as_view(), name='CDNX_products_productfinalattributes_delete'),
    url(r'^productfinalattributes/(?P<pk>\w+)/sublist$', ProductFinalAttributeSubList.as_view(), name='CDNX_products_productfinalattributes_sublist'),
    url(r'^productfinalattributes/(?P<cpk>\w+)/sublist/add$', ProductFinalAttributeCreateModal.as_view(), name='CDNX_products_productfinalattributes_sublist_add'),
    url(r'^productfinalattributes/(?P<cpk>\w+)/sublist/addmodal$', ProductFinalAttributeCreateModal.as_view(), name='CDNX_products_productfinalattributes_sublist_addmodal'),

    url(r'^productfinalattributes/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', ProductFinalAttributeDetailsModal.as_view(), name='CDNX_products_productfinalattributes_sublist_details'),
    url(r'^productfinalattributes/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', ProductFinalAttributeUpdateModal.as_view(), name='CDNX_products_productfinalattributes_sublist_edit'),
    url(r'^productfinalattributes/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', ProductFinalAttributeUpdateModal.as_view(), name='CDNX_products_productfinalattributes_sublist_editmodal'),
    url(r'^productfinalattributes/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', ProductFinalAttributeDelete.as_view(), name='CDNX_products_productfinalattributes_sublist_delete'),

    url(r'^productfeatures$', ProductFeatureList.as_view(), name='CDNX_products_productfeatures_list'),
    url(r'^productfeatures/(?P<pk>\w+)/edit$', ProductFeatureUpdate.as_view(), name='CDNX_products_productfeatures_edit'),
    url(r'^productfeatures/(?P<pk>\w+)/editmodal$', ProductFeatureUpdateModal.as_view(), name='CDNX_products_productfeatures_editmodal'),
    url(r'^productfeatures/(?P<pk>\w+)/delete$', ProductFeatureDelete.as_view(), name='CDNX_products_productfeatures_delete'),
    url(r'^productfeatures/(?P<pk>\w+)/sublist$', ProductFeatureSubList.as_view(), name='CDNX_products_productfeatures_sublist'),
    url(r'^productfeatures/(?P<cpk>\w+)/sublist/add$', ProductFeatureCreateModal.as_view(), name='CDNX_products_productfeatures_sublist_add'),
    url(r'^productfeatures/(?P<cpk>\w+)/sublist/addmodal$', ProductFeatureCreateModal.as_view(), name='CDNX_products_productfeatures_sublist_addmodal'),
    url(r'^productfeatures/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', ProductFeatureDetailsModal.as_view(), name='CDNX_products_productfeatures_sublist_details'),
    url(r'^productfeatures/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', ProductFeatureUpdateModal.as_view(), name='CDNX_products_productfeatures_sublist_edit'),
    url(r'^productfeatures/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', ProductFeatureUpdateModal.as_view(), name='CDNX_products_productfeatures_sublist_editmodal'),
    url(r'^productfeatures/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', ProductFeatureDelete.as_view(), name='CDNX_products_productfeatures_sublist_delete'),

    url(r'^productuniques$', ProductUniqueList.as_view(), name='CDNX_products_productuniques_list'),
    url(r'^productuniques/code/(?P<search>[\w\W]+|\*)$', ProductUniqueCodeForeign.as_view(), name='CDNX_products_productuniquescode_foreign'),
    url(r'^productuniques/(?P<pk>\w+)/edit$', ProductUniqueUpdate.as_view(), name='CDNX_products_productuniques_edit'),
    url(r'^productuniques/(?P<pk>\w+)/editmodal$', ProductUniqueUpdateModal.as_view(), name='CDNX_products_productuniques_editmodal'),
    url(r'^productuniques/(?P<pk>\w+)/delete$', ProductUniqueDelete.as_view(), name='CDNX_products_productuniques_delete'),
    url(r'^productuniques/(?P<pk>\w+)/sublist$', ProductUniqueSubList.as_view(), name='CDNX_products_productuniques_sublist'),
    url(r'^productuniques/(?P<cpk>\w+)/sublist/add$', ProductUniqueCreateModal.as_view(), name='CDNX_products_productuniques_sublist_add'),
    url(r'^productuniques/(?P<cpk>\w+)/sublist/addmodal$', ProductUniqueCreateModal.as_view(), name='CDNX_products_productuniques_sublist_addmodal'),
    url(r'^productuniques/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', ProductUniqueDetailsModal.as_view(), name='CDNX_products_productuniques_sublist_details'),
    url(r'^productuniques/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', ProductUniqueUpdateModal.as_view(), name='CDNX_products_productuniques_sublist_edit'),
    url(r'^productuniques/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', ProductUniqueUpdateModal.as_view(), name='CDNX_products_productuniques_sublist_editmodal'),
    url(r'^productuniques/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', ProductUniqueDelete.as_view(), name='CDNX_products_productuniques_sublist_delete'),
    url(r'^productuniques/foreign/(?P<search>[\w\W]+|\*)$', ProductUniqueForeign.as_view(), name='CDNX_products_productunique_foreign'),


    url(r'^flagshipproducts$', FlagshipProductList.as_view(), name='CDNX_products_flagshipproducts_list'),
    url(r'^flagshipproducts/add$', FlagshipProductCreate.as_view(), name='CDNX_products_flagshipproducts_add'),
    url(r'^flagshipproducts/addmodal$', FlagshipProductCreateModal.as_view(), name='CDNX_products_flagshipproducts_addmodal'),
    url(r'^flagshipproducts/(?P<pk>\w+)/edit$', FlagshipProductUpdate.as_view(), name='CDNX_products_flagshipproducts_edit'),
    url(r'^flagshipproducts/(?P<pk>\w+)/editmodal$', FlagshipProductUpdateModal.as_view(), name='CDNX_products_flagshipproducts_editmodal'),
    url(r'^flagshipproducts/(?P<pk>\w+)/delete$', FlagshipProductDelete.as_view(), name='CDNX_products_flagshipproducts_delete'),

    url(r'^listproducts/(?P<type>\w+)/(?P<pk>[0-9]+)$', ListProducts.as_view(), name='CDNX_products_list_products'),
    url(r'^listproductsbase/(?P<type>\w+)/(?P<pk>[0-9]+)$', ListProductsBase.as_view(), name='CDNX_products_list_products_base'),

    url(r'^groupvaluefeatures$', GroupValueFeatureList.as_view(), name='CDNX_products_GroupValueFeatures_list'),
    url(r'^groupvaluefeatures/add$', GroupValueFeatureCreate.as_view(), name='CDNX_products_GroupValueFeatures_add'),
    url(r'^groupvaluefeatures/addmodal$', GroupValueFeatureCreateModal.as_view(), name='CDNX_products_GroupValueFeatures_addmodal'),
    url(r'^groupvaluefeatures/(?P<pk>\w+)$', GroupValueFeatureDetails.as_view(), name='CDNX_products_GroupValueFeatures_details'),
    url(r'^groupvaluefeatures/(?P<pk>[0-9]+)/edit$', GroupValueFeatureUpdate.as_view(), name='CDNX_products_GroupValueFeatures_edit'),
    url(r'^groupvaluefeatures/(?P<pk>\w+)/editmodal$', GroupValueFeatureUpdateModal.as_view(), name='CDNX_products_GroupValueFeatures_editmodal'),
    url(r'^groupvaluefeatures/(?P<pk>\w+)/delete$', GroupValueFeatureDelete.as_view(), name='CDNX_products_GroupValueFeatures_delete'),

    url(r'^groupvalueattributes$', GroupValueAttributeList.as_view(), name='CDNX_products_GroupValueAttributes_list'),
    url(r'^groupvalueattributes/add$', GroupValueAttributeCreate.as_view(), name='CDNX_products_GroupValueAttributes_add'),
    url(r'^groupvalueattributes/addmodal$', GroupValueAttributeCreateModal.as_view(), name='CDNX_products_GroupValueAttributes_addmodal'),
    url(r'^groupvalueattributes/(?P<pk>\w+)$', GroupValueAttributeDetails.as_view(), name='CDNX_products_GroupValueAttributes_details'),
    url(r'^groupvalueattributes/(?P<pk>[0-9]+)/edit$', GroupValueAttributeUpdate.as_view(), name='CDNX_products_GroupValueAttributes_edit'),
    url(r'^groupvalueattributes/(?P<pk>\w+)/editmodal$', GroupValueAttributeUpdateModal.as_view(), name='CDNX_products_GroupValueAttributes_editmodal'),
    url(r'^groupvalueattributes/(?P<pk>\w+)/delete$', GroupValueAttributeDelete.as_view(), name='CDNX_products_GroupValueAttributes_delete'),

    url(r'^groupvaluefeaturespecials$', GroupValueFeatureSpecialList.as_view(), name='CDNX_products_GroupValueFeatureSpecials_list'),
    url(r'^groupvaluefeaturespecials/add$', GroupValueFeatureSpecialCreate.as_view(), name='CDNX_products_GroupValueFeatureSpecials_add'),
    url(r'^groupvaluefeaturespecials/addmodal$', GroupValueFeatureSpecialCreateModal.as_view(), name='CDNX_products_GroupValueFeatureSpecials_addmodal'),
    url(r'^groupvaluefeaturespecials/(?P<pk>\w+)$', GroupValueFeatureSpecialDetails.as_view(), name='CDNX_products_GroupValueFeatureSpecials_details'),
    url(r'^groupvaluefeaturespecials/(?P<pk>[0-9]+)/edit$', GroupValueFeatureSpecialUpdate.as_view(), name='CDNX_products_GroupValueFeatureSpecials_edit'),
    url(r'^groupvaluefeaturespecials/(?P<pk>\w+)/editmodal$', GroupValueFeatureSpecialUpdateModal.as_view(), name='CDNX_products_GroupValueFeatureSpecials_editmodal'),
    url(r'^groupvaluefeaturespecials/(?P<pk>\w+)/delete$', GroupValueFeatureSpecialDelete.as_view(), name='CDNX_products_GroupValueFeatureSpecials_delete'),


    url(r'^optionvaluefeatures$', OptionValueFeatureList.as_view(), name='CDNX_products_OptionValueFeatures_list'),
    url(r'^optionvaluefeatures/(?P<gpk>\w+)/edit$', OptionValueFeatureUpdate.as_view(), name='CDNX_products_OptionValueFeatures_edit'),
    url(r'^optionvaluefeatures/(?P<gpk>\w+)/editmodal$', OptionValueFeatureUpdateModal.as_view(), name='CDNX_products_OptionValueFeatures_editmodal'),
    url(r'^optionvaluefeatures/(?P<gpk>\w+)/delete$', OptionValueFeatureDelete.as_view(), name='CDNX_products_OptionValueFeatures_delete'),
    url(r'^optionvaluefeatures/(?P<pk>\w+)/sublist$', OptionValueFeatureSubList.as_view(), name='CDNX_products_OptionValueFeatures_sublist'),
    url(r'^optionvaluefeatures/(?P<pk>\w+)/sublistm$', OptionValueFeatureSubListModal.as_view(), name='CDNX_products_OptionValueFeatures_sublist_modal'),
    url(r'^optionvaluefeatures/(?P<gpk>[0-9]+)/sublist/add$', OptionValueFeatureCreateModal.as_view(), name='CDNX_products_optionv2alues_sublist_add'),
    url(r'^optionvaluefeatures/(?P<gpk>[0-9]+)/sublistm/add$', OptionValueFeatureCreateModal.as_view(), name='CDNX_products_optionv2alues_sublist_add2'),
    url(r'^optionvaluefeatures/(?P<gpk>[0-9]+)/sublist/addmodal$', OptionValueFeatureCreateModal.as_view(), name='CDNX_products_optionv2alues_sublist_addmodal'),
    url(r'^optionvaluefeatures/(?P<cpk>[0-9]+)/sublist/(?P<pk>[0-9]+)$', OptionValueFeatureDetailsModal.as_view(), name='CDNX_products_OptionValueFeatures_sublist_details'),
    url(r'^optionvaluefeatures/(?P<cpk>[0-9]+)/sublistm/(?P<pk>[0-9]+)$', OptionValueFeatureDetailsModal.as_view(), name='CDNX_products_OptionValueFeatures_sublist_details'),
    url(r'^optionvaluefeatures/(?P<cpk>[0-9]+)/sublistm/(?P<pk>[0-9]+)/edit$', OptionValueFeatureUpdateModal.as_view(), name='CDNX_products_OptionValueFeatures_sublist_editmodal'),
    url(r'^optionvaluefeatures/(?P<cpk>[0-9]+)/sublist/(?P<pk>[0-9]+)/editmodal$', OptionValueFeatureUpdateModal.as_view(), name='CDNX_products_OptionValueFeatures_sublist_editmodal'),
    url(r'^optionvaluefeatures/(?P<cpk>[0-9]+)/sublist/(?P<pk>[0-9]+)/delete$', OptionValueFeatureDelete.as_view(), name='CDNX_products_OptionValueFeatures_sublist_delete'),
    url(r'^optionvaluefeatures/foreign/(?P<search>[\w\W]+|\*)$', OptionValueFeatureForeign.as_view(), name='CDNX_products_OptionValueFeatures_foreign'),

    url(r'^optionvalueattributes$', OptionValueAttributeList.as_view(), name='CDNX_products_OptionValueAttributes_list'),
    url(r'^optionvalueattributes/(?P<gpk>\w+)/edit$', OptionValueAttributeUpdate.as_view(), name='CDNX_products_OptionValueAttributes_edit'),
    url(r'^optionvalueattributes/(?P<gpk>\w+)/editmodal$', OptionValueAttributeUpdateModal.as_view(), name='CDNX_products_OptionValueAttributes_editmodal'),
    url(r'^optionvalueattributes/(?P<gpk>\w+)/delete$', OptionValueAttributeDelete.as_view(), name='CDNX_products_OptionValueAttributes_delete'),
    url(r'^optionvalueattributes/(?P<pk>\w+)/sublist$', OptionValueAttributeSubList.as_view(), name='CDNX_products_OptionValueAttributes_sublist'),
    url(r'^optionvalueattributes/(?P<pk>\w+)/sublistm$', OptionValueAttributeSubListModal.as_view(), name='CDNX_products_OptionValueAttributes_sublist_modal'),
    url(r'^optionvalueattributes/(?P<gpk>[0-9]+)/sublist/add$', OptionValueAttributeCreateModal.as_view(), name='CDNX_products_optionv2alues_sublist_add'),
    url(r'^optionvalueattributes/(?P<gpk>[0-9]+)/sublistm/add$', OptionValueAttributeCreateModal.as_view(), name='CDNX_products_optionv2alues_sublist_add2'),
    url(r'^optionvalueattributes/(?P<gpk>[0-9]+)/sublist/addmodal$', OptionValueAttributeCreateModal.as_view(), name='CDNX_products_optionv2alues_sublist_addmodal'),
    url(r'^optionvalueattributes/(?P<cpk>[0-9]+)/sublist/(?P<pk>[0-9]+)$', OptionValueAttributeDetailsModal.as_view(), name='CDNX_products_OptionValueAttributes_sublist_details'),
    url(r'^optionvalueattributes/(?P<cpk>[0-9]+)/sublistm/(?P<pk>[0-9]+)$', OptionValueAttributeDetailsModal.as_view(), name='CDNX_products_OptionValueAttributes_sublist_details'),
    url(r'^optionvalueattributes/(?P<cpk>[0-9]+)/sublistm/(?P<pk>[0-9]+)/edit$', OptionValueAttributeUpdateModal.as_view(), name='CDNX_products_OptionValueAttributes_sublist_editmodal'),
    url(r'^optionvalueattributes/(?P<cpk>[0-9]+)/sublist/(?P<pk>[0-9]+)/editmodal$', OptionValueAttributeUpdateModal.as_view(), name='CDNX_products_OptionValueAttributes_sublist_editmodal'),
    url(r'^optionvalueattributes/(?P<cpk>[0-9]+)/sublist/(?P<pk>[0-9]+)/delete$', OptionValueAttributeDelete.as_view(), name='CDNX_products_OptionValueAttributes_sublist_delete'),
    url(r'^optionvalueattributes/foreign/(?P<search>[\w\W]+|\*)$', OptionValueAttributeForeign.as_view(), name='CDNX_products_OptionValueAttributes_foreign'),

    url(r'^optionvaluefeaturespecials$', OptionValueFeatureSpecialList.as_view(), name='CDNX_products_OptionValueFeatureSpecials_list'),
    url(r'^optionvaluefeaturespecials/(?P<gpk>\w+)/edit$', OptionValueFeatureSpecialUpdate.as_view(), name='CDNX_products_OptionValueFeatureSpecials_edit'),
    url(r'^optionvaluefeaturespecials/(?P<gpk>\w+)/editmodal$', OptionValueFeatureSpecialUpdateModal.as_view(), name='CDNX_products_OptionValueFeatureSpecials_editmodal'),
    url(r'^optionvaluefeaturespecials/(?P<gpk>\w+)/delete$', OptionValueFeatureSpecialDelete.as_view(), name='CDNX_products_OptionValueFeatureSpecials_delete'),
    url(r'^optionvaluefeaturespecials/(?P<pk>\w+)/sublist$', OptionValueFeatureSpecialSubList.as_view(), name='CDNX_products_OptionValueFeatureSpecials_sublist'),
    url(r'^optionvaluefeaturespecials/(?P<pk>\w+)/sublistm$', OptionValueFeatureSpecialSubListModal.as_view(), name='CDNX_products_OptionValueFeatureSpecials_sublist_modal'),
    url(r'^optionvaluefeaturespecials/(?P<gpk>[0-9]+)/sublist/add$', OptionValueFeatureSpecialCreateModal.as_view(), name='CDNX_products_optionv2alues_sublist_add'),
    url(r'^optionvaluefeaturespecials/(?P<gpk>[0-9]+)/sublistm/add$', OptionValueFeatureSpecialCreateModal.as_view(), name='CDNX_products_optionv2alues_sublist_add2'),
    url(r'^optionvaluefeaturespecials/(?P<gpk>[0-9]+)/sublist/addmodal$', OptionValueFeatureSpecialCreateModal.as_view(), name='CDNX_products_optionv2alues_sublist_addmodal'),
    url(r'^optionvaluefeaturespecials/(?P<cpk>[0-9]+)/sublist/(?P<pk>[0-9]+)$', OptionValueFeatureSpecialDetailsModal.as_view(), name='CDNX_products_OptionValueFeatureSpecials_sublist_details'),
    url(r'^optionvaluefeaturespecials/(?P<cpk>[0-9]+)/sublistm/(?P<pk>[0-9]+)$', OptionValueFeatureSpecialDetailsModal.as_view(), name='CDNX_products_OptionValueFeatureSpecials_sublist_details'),
    url(r'^optionvaluefeaturespecials/(?P<cpk>[0-9]+)/sublistm/(?P<pk>[0-9]+)/edit$', OptionValueFeatureSpecialUpdateModal.as_view(), name='CDNX_products_OptionValueFeatureSpecials_sublist_editmodal'),
    url(r'^optionvaluefeaturespecials/(?P<cpk>[0-9]+)/sublist/(?P<pk>[0-9]+)/editmodal$', OptionValueFeatureSpecialUpdateModal.as_view(), name='CDNX_products_OptionValueFeatureSpecials_sublist_editmodal'),
    url(r'^optionvaluefeaturespecials/(?P<cpk>[0-9]+)/sublist/(?P<pk>[0-9]+)/delete$', OptionValueFeatureSpecialDelete.as_view(), name='CDNX_products_OptionValueFeatureSpecials_sublist_delete'),
    url(r'^optionvaluefeaturespecials/foreign/(?P<search>[\w\W]+|\*)$', OptionValueFeatureSpecialForeign.as_view(), name='CDNX_products_OptionValueFeatureSpecials_foreign'),

    url(r'^productfinaloptions$', ProductFinalOptionList.as_view(), name='CDNX_products_productfinaloptions_list'),
    url(r'^productfinaloptions/add$', ProductFinalOptionCreate.as_view(), name='CDNX_products_productfinaloptions_add'),
    url(r'^productfinaloptions/addmodal$', ProductFinalOptionCreateModal.as_view(), name='CDNX_products_productfinaloptions_addmodal'),
    url(r'^productfinaloptions/(?P<pk>\w+)$', ProductFinalOptionDetails.as_view(), name='CDNX_products_productfinaloptions_details'),
    url(r'^productfinaloptions/(?P<pk>\w+)/edit$', ProductFinalOptionUpdate.as_view(), name='CDNX_products_productfinaloptions_edit'),
    url(r'^productfinaloptions/(?P<pk>\w+)/editmodal$', ProductFinalOptionUpdateModal.as_view(), name='CDNX_products_productfinaloptions_editmodal'),
    url(r'^productfinaloptions/(?P<pk>\w+)/delete$', ProductFinalOptionDelete.as_view(), name='CDNX_products_productfinaloptions_delete'),
    url(r'^productfinaloptions/(?P<pk>\w+)/sublist$', ProductFinalOptionSubList.as_view(), name='CDNX_products_productfinaloptions_sublist'),
    url(r'^productfinaloptions/(?P<cpk>\w+)/sublist/add$', ProductFinalOptionCreateModal.as_view(), name='CDNX_products_productfinaloptions_sublist_add'),
    url(r'^productfinaloptions/(?P<cpk>\w+)/sublist/addmodal$', ProductFinalOptionCreateModal.as_view(), name='CDNX_products_productfinaloptions_sublist_addmodal'),
    url(r'^productfinaloptions/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', ProductFinalOptionDetailModal.as_view(), name='CDNX_products_productfinaloptions_sublist_details'),
    url(r'^productfinaloptions/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', ProductFinalOptionUpdateModal.as_view(), name='CDNX_products_productfinaloptions_sublist_edit'),
    url(r'^productfinaloptions/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', ProductFinalOptionUpdateModal.as_view(), name='CDNX_products_productfinaloptions_sublist_editmodal'),
    url(r'^productfinaloptions/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', ProductFinalOptionDelete.as_view(), name='CDNX_products_productfinaloptions_sublist_delete'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-01-25 07:25
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_products', '0006_auto_20180118_1126'),
    ]

    operations = [
        migrations.AddField(
            model_name='product',
            name='caducable',
            field=models.BooleanField(default=False, verbose_name='Caducable'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-01-23 12:16
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_products', '0006_auto_20180118_1126'),
    ]

    operations = [
        migrations.AddField(
            model_name='productunique',
            name='stock_locked',
            field=models.FloatField(default=0, editable=False, verbose_name='Stock locked'),
        ),
        migrations.AddField(
            model_name='productunique',
            name='stock_original',
            field=models.FloatField(default=0, editable=False, verbose_name='Stock original'),
        ),
        migrations.AlterField(
            model_name='productunique',
            name='stock_real',
            field=models.FloatField(default=0, editable=False, verbose_name='Stock real'),
        ),
    ]
